!
! calls to ReadSurfaceField from:
!   diffusion.F90
!   dry_deposition_cbm4.F90
!   sources_sinks_cbm4.F90
!

module meteofiles

  use Dims, only : nregions
  use TMkit, only : TtmMeteo
  use TMkit, only : TllGridInfo, TLevelInfo

  implicit none
  
  
  ! --- in/out ----------------------------------------

  private

  public  ::  MeteoFiles_Init, MeteoFiles_Done
  public  ::  tmmd
  public  ::  sourcekey
  public  ::  lli, lli_1x1
  public  ::  levi, levi_ec, levi_tm60
  public  ::  ReadSurfaceField

  
  ! --- const ------------------------------------------
  
  ! module name
  character(len=*), parameter  ::  mname = 'MeteoFiles'
  

  ! --- local -----------------------------------
  
  ! source key
  character(len=50)            ::  sourcekey(nregions)
  
  ! interface to TM meteo data
  type(TtmMeteo)               ::  tmmd

  ! horizontal grid definitions:
  type(TllGridInfo)            ::  lli(nregions)
  type(TllGridInfo)            ::  lli_1x1
  
  ! vertical grid definition:
  type(TLevelInfo)             ::  levi
  type(TLevelInfo)             ::  levi_ec
  type(TLevelInfo)             ::  levi_tm60



contains


  ! ============================================================

  subroutine MeteoFiles_Init( status )

    use GO, only : TrcFile, Init, Done, ReadRc
    use Dims, only : nregions, region_name
    use Dims, only : xbeg, dx, xref, im
    use Dims, only : ybeg, dy, yref, jm
    use Dims, only : echlevs, lme, a_ec, b_ec
    use TMkit, only : TLevelInfo, Init, Done

    use global_data, only : rcfile, bindir

    ! --- in/out -------------------------------
        
    integer, intent(out)         ::  status
    
    ! --- const ----------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/MeteoFiles_Init'
    
    ! --- local ----------------------------
    
    type(TrcFile)      ::  rcF
    integer            ::  n

    ! --- begin ----------------------------
    
    ! setup interface to TM meteo:
    call Init( tmmd, trim(bindir)//'/tmm_getmeteo', rcfile, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
    
    ! read settings from rcfile
    call Init( rcF, rcfile )
    do n = 1, nregions
      call ReadRc( rcF, 'tmm.sourcekey.'//region_name(n), sourcekey(n) )
    end do
    call Done( rcF )
    
    !
    ! setup horizontal grids
    !
    
    ! define mid of first west/south cells, spacing, and size:
    do n = 1, nregions
      call Init( lli(n), xbeg(n)+dx/xref(n)/2.0, real(dx)/xref(n), im(n), &
                         ybeg(n)+dy/yref(n)/2.0, real(dy)/yref(n), jm(n), status )
      if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
    end do
    !
    call Init( lli_1x1, -179.5, 1.0, 360, -89.5, 1.0, 180, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
    
    !
    ! level definition
    !
    
    ! setup parent level definition:
    call Init( levi_ec, lme, a_ec, b_ec, status )          ! ecmwf levels
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if

    ! setup level definition:
    call Init( levi, levi_ec, echlevs, status )   ! tm half level selection
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if

    ! all levels ...    
    call Init( levi_tm60, 'tm60', status )          ! TM 60 levels
    
    !
    ! done
    !

    ! ok
    status = 0

  end subroutine MeteoFiles_Init


  ! ***


  subroutine MeteoFiles_Done( status )

    use Dims, only : nregions  
    use TMkit, only : Done

    ! --- in/out -------------------------------
        
    integer, intent(out)         ::  status
    
    ! --- const ----------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/MeteoFiles_Done'
    
    ! --- local ----------------------------------
    
    integer           ::  n
    
    ! --- begin ----------------------------

    ! interface to TM meteo:
    call Done( tmmd, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
    
    ! horizontal grids:
    do n = 1, nregions
      call Done( lli(n), status )
      if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
    end do
    !
    call Done( lli_1x1, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
    
    ! done 60level definition:
    call Done( levi_tm60, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if

    ! done parent level definition:
    call Done( levi_ec, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if

    ! level definition:
    call Done( levi, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if

    ! ok
    status = 0

  end subroutine MeteoFiles_Done


  ! ===================================================================
  

  subroutine ReadSurfaceField( field, fieldname, idater, dhour, status )
  
    use TMKit, only : TDate, NewDate, IncrDate, operator(+)
    use TMKit, only : ReadField
    use TMKit, only : PrintDate

    ! --- in/out ------------------------------------
    
    real, intent(out)               ::  field(:,:)
    character(len=*), intent(in)    ::  fieldname
    integer, intent(in)             ::  idater(6)
    integer, intent(in)             ::  dhour
    integer, intent(out)            ::  status
    
    ! --- const -------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/ReadSurfaceField'
    
    ! --- local ------------------------------------

    type(TDate)             ::  t1, t2
    integer                 ::  fcday

    ! --- begin -----------------------------------

    ! set times to t1+[0,6]
    t1 = NewDate( time6=idater )
    t2 = t1 + IncrDate( hour=dhour )
    fcday = 0
    
    write (*,'(a,": reading ",a)') rname, fieldname
    call PrintDate( rname//': t1 = ', t1 )
    call PrintDate( rname//': t2 = ', t2 )
    
    ! read surface field from data base:
    call ReadField( tmmd, sourcekey(1), fieldname, t1, t2, fcday, &
                          lli_1x1, 'n', field, status )
    if (status/=0) then; write (*,'("ERROR in ",a)') rname; return; end if
    
    ! ok
    status = 0

  end subroutine ReadSurfaceField


end module meteofiles


