!#################################################################
!
! routine to perform main processing and      
! handles time ordering of model processes    
!
!### macro's #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module modelIntegration

  use GO, only : gol, goErr
  
  implicit none
  
  
  ! --- in/out -----------------------------

  private

  public :: Proces_Init, Proces_Done
  public :: Proces_Region
  
  public :: advect_apply, convec_diffusion_apply
 
  
  ! --- const ----------------------------------
  
  ! module name
  character(len=*), parameter  ::  mname = 'ModelIntegration'
  

  ! --- var ---------------------------------
  
  logical    ::  advect_apply=.true.
  logical    ::  chemistry_apply=.true.
  logical    ::  source_apply=.true.
  logical    ::  convec_apply=.true.
  logical    ::  wet_removal_apply=.true.
  logical    ::  convec_diffusion_apply=.true.
  
  logical    ::  meteo_read=.true.
  logical    ::  meteo_read_temper=.true.
  logical    ::  meteo_read_cloud=.true.
  logical    ::  meteo_read_albedo=.true.
  logical    ::  meteo_read_ocean = .false.
  logical    ::  meteo_read_ci    = .false.
  logical    ::  meteo_read_sps   = .false.
  logical    ::  meteo_read_sst   = .false.
  logical    ::  meteo_read_gph=.true.
  logical    ::  meteo_read_convec=.true.
  

contains

  
  ! =====================================================
  
  
  subroutine Proces_Init( status )
  
    use Binas      , only : p_global
    use GO         , only : TrcFile, Init, Done, ReadRc
    use dims       , only : nregions
    use global_data, only : rcfile
    use global_data, only : use_convec
    use Meteo      , only : use_clouds, new_cloud
    use Meteo      , only : sp1_dat, sp2_dat, sp_dat, spm_dat
    use Meteo      , only : mfu_dat, mfv_dat, mfw_dat
    use Meteo      , only :  pu_dat,  pv_dat
    use Meteo      , only : temper_dat, humid_dat
    use Meteo      , only : lwc_dat, iwc_dat, cc_dat, cco_dat, ccu_dat
    use Meteo      , only : gph_dat, omega_dat
    use Meteo      , only : entu_dat, entd_dat, detu_dat, detd_dat
    use Meteo      , only : oro_sfc, lsmask_sfc
    use Meteo      , only : albedo_sfc, sr_ecm_sfc, sr_ols_sfc
    use Meteo      , only : sp_sfc, ci_sfc, sst_sfc
    use Meteo      , only : u10m_sfc, v10m_sfc
    use Meteo      , only : slhf_sfc, sshf_sfc, ss_sfc, cp_sfc, lsp_sfc
    use Meteo      , only : sd_sfc, swvl1_sfc, src_sfc, d2m_sfc, t2m_sfc, ssr_sfc
    use Meteo      , only : tv_sfc, cvl_sfc, cvh_sfc
    use Meteo      , only : Set, CoarseMeteo_Init
    use Meteo      , only : Meteo_Alloc
    use redgridZoom, only : grid_reduced

    use Advect       , only : Advect_Init
    use Convection   , only : Convection_Init
    use Diffusion    , only : Diffusion_Init
    use Chemistry    , only : Chemistry_Init
    use emission    , only : Emis_Init
    use Sources_Sinks, only : Sources_Sinks_Init
    
    ! --- in/out --------------------------------
    
    integer, intent(out)    ::  status
    
    ! --- const -------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Proces_Init'
    
    ! --- local ---------------------------------
    
    type(TrcFile)     ::  rcF
    integer           ::  region
    character(len=256)::  sourcekey_1x1
    logical           ::  use_extra_surf=.true. ! use extra surface fields not in NCEP archives currently
    
    ! --- begin ---------------------------------
  
    !
    ! read from rcfile
    !
    
    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'proces.advection'        , advect_apply, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'proces.advection.reduced', grid_reduced, status )
    IF_NOTOK_RETURN(status=1)
    
    call ReadRc( rcF, 'proces.convec', convec_apply, status )
    IF_NOTOK_RETURN(status=1)
    if ( convec_apply ) then
      call ReadRc( rcF, 'proces.convec.diffusion', convec_diffusion_apply, status )
      IF_NOTOK_RETURN(status=1)
    else
      wet_removal_apply = .false.
      convec_diffusion_apply   = .false.
    end if
    ! flag 'use_convec' is used in global_data to allocate some fields
    use_convec = convec_apply

    call ReadRc( rcF, 'proces.chemistry', chemistry_apply, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'proces.source'   , source_apply, status    )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'proces.wet_removal', wet_removal_apply, status )
    IF_NOTOK_RETURN(status=1)
    if ( wet_removal_apply .and. (.not. source_apply) ) then
      write (gol,'("wet removal requires processes source ...")'); call goErr
      write (*,'("ERROR in ",a)') rname; status=1; return
    end if
    if ( wet_removal_apply .and. (.not. convec_apply) ) then
      write (gol,'("wet removal requires processes convec ...")'); call goErr
      write (*,'("ERROR in ",a)') rname; status=1; return
    end if

    call ReadRc( rcF, 'tmm.sourcekey.glb1x1.*', sourcekey_1x1, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.*'      , meteo_read, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.temper' , meteo_read_temper, status, default=meteo_read )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.cloud' , meteo_read_cloud , status, default=meteo_read )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.albedo', meteo_read_albedo, status, default=meteo_read )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.ci'    , meteo_read_ci    , status, default=meteo_read_ci )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.sps'   , meteo_read_sps   , status, default=meteo_read_sps )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.sst'   , meteo_read_sst   , status, default=meteo_read_sst )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.gph'   , meteo_read_gph   , status, default=meteo_read )
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'meteo.read.convec', meteo_read_convec, status, default=meteo_read )
    IF_ERROR_RETURN(status=1)

    ! backwards compatible ...
    call ReadRc( rcF, 'meteo.read.ocean', meteo_read_ocean  , status, default=.false. )
    IF_ERROR_RETURN(status=1)
    if ( meteo_read_ocean ) then
      meteo_read_ci  = .true.
      meteo_read_sst = .true.
      meteo_read_sps = .true.
    end if
    
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)
    

    !
    ! select meteo to be used
    !

    ! always use surface pressure (necessary for pressure and mass fields)    
    call Set(  sp_dat, status, used=.true. )   
    call Set( spm_dat, status, used=.true. )
    
    ! process standard meteo ?
    if ( meteo_read ) then

      ! used by advection and diffusion:    
      call Set(  sp_dat, status, used=.true. )
      call Set( sp1_dat, status, used=.true. )
      call Set( sp2_dat, status, used=.true. )
      call Set( mfu_dat, status, used=.true. )
      call Set( mfv_dat, status, used=.true. )
      call Set( mfw_dat, status, used=.true. )
      call Set(  pu_dat, status, used=.true. )
      call Set(  pv_dat, status, used=.true. )

      ! omega is used for something:
      call Set( omega_dat, status, used=.true. )

      ! used by diffusion
      call Set( lsmask_sfc, status, used=.true. )
      call Set( sr_ecm_sfc, status, used=.true. )
      call Set( sr_ols_sfc, status, used=.true. )
      call Set(   u10m_sfc, status, used=.true. )
      call Set(   v10m_sfc, status, used=.true. )
      call Set(   sshf_sfc, status, used=.true. )
      call Set(   slhf_sfc, status, used=.true. )
      call Set(     ss_sfc, status, used=.true. )

      ! used by wet deposition ?
      call Set(    lsp_sfc, status, used=.true. )
      call Set(     cp_sfc, status, used=.true. )

      ! other surface fields
      call Set(    sd_sfc, status, used=.true. )
      call Set( swvl1_sfc, status, used=.true. )
      call Set(   ssr_sfc, status, used=.true. )
      call Set(   t2m_sfc, status, used=.true. )

      IF(sourcekey_1x1(1:4) == 'ncep') use_extra_surf=.false.

      call Set(   src_sfc, status, used=use_extra_surf )
      call Set(   d2m_sfc, status, used=use_extra_surf )
      call Set(    tv_sfc, status, used=use_extra_surf )
      call Set(   cvl_sfc, status, used=use_extra_surf )
      call Set(   cvh_sfc, status, used=use_extra_surf )

    end if
    
    ! process convection fields ?
    if ( meteo_read_convec ) then
      ! entrements/detrements
      call Set( entu_dat, status, used=.true. )
      call Set( entd_dat, status, used=.true. )
      call Set( detu_dat, status, used=.true. )
      call Set( detd_dat, status, used=.true. )
      ! omega is used to estimate vertical wind; computed from mfw :
      call Set( omega_dat, status, used=.true. )
      call Set(   mfw_dat, status, used=.true. )
    end if

    ! process temperature meteo ?
    if ( meteo_read_temper ) then
      call Set(  temper_dat, status, used=.true. )
    end if

    ! process meteo for gph ?
    if ( meteo_read_gph ) then
      call Set(  temper_dat, status, used=.true. )
      call Set(   humid_dat, status, used=.true. )
      call Set(     oro_sfc, status, used=.true. )
      call Set(     gph_dat, status, used=.true. )
    end if

    ! declare/read/use cloud data
    if ( meteo_read_cloud ) then
      ! flag individual cloud fields:
      call Set(  lwc_dat, status, used=.true. )
      call Set(  iwc_dat, status, used=.true. )
      call Set(   cc_dat, status, used=.true. )
      call Set(  cco_dat, status, used=.true. )
      call Set(  ccu_dat, status, used=.true. )
      ! used by photolyis: clouds used at all ? new cloud fields are read ?
      use_clouds = .true.
      new_cloud  = .true.
    else
      ! used by photolyis: clouds used at all ? new cloud fields are read ?
      use_clouds = .false.
      new_cloud  = .false.
    end if

    ! used by photolysis ?
    if ( meteo_read_albedo ) then
      call Set( albedo_sfc, status, used=.true. )
    end if

    ! used for ocean emis:
    if ( meteo_read_ci ) then
      call Set(  ci_sfc, status, used=.true. )
    end if
    if ( meteo_read_sst ) then
      call Set( sst_sfc, status, used=.true. )
    end if
    if ( meteo_read_sps ) then
      call Set(  sp_sfc, status, used=.true. )
    end if


    ! 
    ! init processes
    !
    
    ! NOTE: these inits should select the appropriate meteo 
    
    ! init advection ?
    if ( advect_apply ) then
      call Advect_Init( status )
      IF_NOTOK_RETURN(status=1)
    end if
    
    ! init convec ?
    if ( convec_apply ) then
      call Convection_Init( wet_removal_apply, convec_diffusion_apply, status )
      IF_NOTOK_RETURN(status=1)
    end if

    ! init diffusion ?
    if ( convec_diffusion_apply ) then
      call Diffusion_Init( status )
      IF_NOTOK_RETURN(status=1)
    end if

    ! init chemistry ?
    if ( chemistry_apply ) then
      call Chemistry_Init( status )
      IF_NOTOK_RETURN(status=1)
    end if
    
    ! init sources and sinks ?
    if ( source_apply ) then
      call Sources_Sinks_Init( status )
      !call Emis_Init( status )  !WP! not yet, but for future
      IF_NOTOK_RETURN(status=1)
    end if
    
    
    !
    ! allocate meteo
    !

    ! allocate used meteo
    call Meteo_Alloc( status )
    IF_NOTOK_RETURN(status=1)
    
    ! setup coarsened meteo
    call CoarseMeteo_Init( status )
    IF_NOTOK_RETURN(status=1)


    !
    ! default values
    !

    ! Fill default values for surface pressure;
    ! might be required for mass fields used in 
    ! init routines I am not aware of ..
    do region = 1, nregions
      sp_dat(region)%data = p_global
    end do
    
    
    !
    ! done
    !
    
    ! ok
    status = 0

  end subroutine Proces_Init
  
  
  ! ***
  
     
  subroutine Proces_Done( status )
  
    use Meteo    , only : CoarseMeteo_Done
    use Chemistry, only : Chemistry_Done

    use Advect       , only : Advect_Done
    use Convection   , only : Convection_Done
    use Diffusion    , only : Diffusion_Done
    use Chemistry    , only : Chemistry_Done
    use Sources_Sinks, only : Sources_Sinks_Done
    
    ! --- in/out --------------------------------
    
    integer, intent(out)    ::  status
    
    ! --- const -------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Proces_Done'
    
    ! --- begin ---------------------------------
    
    ! 
    ! init processes
    !
    
    if ( advect_apply ) then
      call Advect_Done( status )
      IF_NOTOK_RETURN(status=1)
    end if
    
    ! done convec ?
    if ( convec_apply ) then
      call Convection_Done( status )
      IF_NOTOK_RETURN(status=1)
    end if

    ! done diffusion ?
    if ( convec_diffusion_apply ) then
      call Diffusion_Done( status )
      IF_NOTOK_RETURN(status=1)
    end if

    if ( chemistry_apply ) then
      call Chemistry_Done( status )
      IF_NOTOK_RETURN(status=1)
    end if
    
    if ( source_apply ) then
      call Sources_Sinks_Done( status )
      !WP! call Emis_Done('', status ) !WP still in sources_sinks for now
      IF_NOTOK_RETURN(status=1)
    end if
    
    
    !
    ! clear meteo
    !

    ! clear coarse meteo
    call CoarseMeteo_Done( status )
    IF_NOTOK_RETURN(status=1)
    
    ! ok
    status = 0

  end subroutine Proces_Done
  
  
  ! ***
  
  
  !
  ! for a given region, performs tref(region)/tref(parent(region)) 
  ! advection steps for each direction x,y,z
  ! including all region's children, granchildren, etc. 
  ! written by patrick berkvens and mike botchev, march-june 1999
  !WP! Also chemistry and sources_sinks are called during advection, 
  !WP! wouter peters, july 2000
  !
  !example (cmk)
  !   XYZ                              ECCEZYX
  !      xyz          eccezyx                  cezyx          xyz          ec
  !         xyzeccezyx       ecxyzzyxec             cezyxxyzec   zyxeccexyz
  ! in this case the operations e and c should be executed in the 
  ! interface region
  ! but not in the core of the zoom region (otherwise double counting)
  ! This results in the most simple algorithm, because you should leave
  ! the DO_STEPS routine AFTER 
  ! (1) finishing all the steps...
  ! (2) XYZ
  ! MK 2000...
  !

  recursive subroutine Proces_Region( region, tr, status )

    use GO          , only : TDate, NewDate
    use dims        , only : tref, revert, ndyn, itaur
    use dims        , only : parent, children
    use dims        , only : statusr => status
    use dims        , only : n_operators
    use datetime    , only : tau2date
    use user_output , only : user_output_step
    use zoom_tools  , only : update_parent
    use advect_tools, only : m2phlb, m2phlb1

    ! --- in/out -------------------------------
    
    integer, intent(in)   ::  region
    type(TDate)           ::  tr(2)
    integer, intent(out)  ::  status

    ! --- const -------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/proces_region'
    
    ! --- local ---------------------------------
    
    integer :: child, i, ichild, tref_, n_children, dtime

    integer       ::  time6(6)
    type(TDate)   ::  tri(2)

    ! --- begin ----------------------------------
    
    ! determine refinement factor with respect to the parent
    tref_ = tref(region)/tref(parent(region))

    !main time step for all processes. note that all timesteps are default/2
    dtime = revert*ndyn/(2*tref(region))
    !e.g. xyzvec  (firts call) and cevzyx (second call)

    do i = 1, tref_

       ! time step: itaur(region)+[0,dtime]
       call tau2date( itaur(region), time6 )
       tri(1) = NewDate( time6=time6 )
       call tau2date( itaur(region)+dtime, time6 )
       tri(2) = NewDate( time6=time6 )

       call do_steps( region, tri, status )
       IF_NOTOK_RETURN(status=1)

       ! CALL advect_region for all the children (IF there are any)

       ichild = 0
       do while ( ichild < children(region,0) )
          ichild = ichild + 1
          child = children(region,ichild)
          call proces_region( child, tri, status )
          IF_NOTOK_RETURN(status=1)
       end do

       ! do the remaining steps if necessary...!
       if ( mod(statusr(region),n_operators) /= 0 ) then
          call do_steps( region, tri, status )
          IF_NOTOK_RETURN(status=1)
       end if
       
       ! update region time.....
       itaur(region) = itaur(region) + dtime

       ! Accumulate output, write info at stations
       ! Note: the edges in the interface may not have been updated: 
       !   this is the task of the parent.
       call user_output_step( region, status )   
       IF_NOTOK_RETURN(status=1)

    end do

    ! update parent using child data ?
    ! compute pressure grid from (changed) air mass:
    if ( region /= 1 ) then
       call update_parent( region )
       call m2phlb( region )
    else
       call m2phlb1( 1 )
    end if

    !if(okdebug) call Par_Check_mass(region,'after proces_region')
    
    ! ok
    status = 0

  end subroutine proces_region


  ! ***
  

  !-------------------------------------------------------------
  ! performs next three (x-, y- and z-) advection steps
  ! written by patrick berkvens and mike botchev, march-june 1999
  !wp! added the call to chemistry. subroutine now does four 
  !wp! steps (c-x-y-z) wp, july 2000
  !mk! changed the order for proper processing of the interfaces.
  !mk! feb-2001
  !-------------------------------------------------------------

  subroutine do_steps( region, tr, status )

    use GO           , only : TDate
    use dims         , only : statusr => status
    use dims         , only : okdebug
    use dims         , only : splitorderzoom, n_operators
    use chemistry    , only : chemie
    use sources_sinks, only : source1,source2
#ifdef with_budgets
    use budget_global, only : budget_transportg,diagbudg,apply_budget_global
    use budget_fine  , only : budget_transport
#endif
    use toolbox,       only : escape_tm
#ifdef MPI 
    use mpi_const,     only : previous_par
#endif
    use ParTools     , only : myid, root
    use ParTools     , only : ntracetloc, ntracet_ar
    use tracer_data  , only : Par_Check_Domain, Par_Check_Mass
    use global_data,   only : mass_dat
    use chem_param,    only : names, ntracet   !CMKTEMP
    use convection,    only : convec
    use advectx,    only : advectxzoom, put_xedges
    use advecty,    only : advectyzoom, put_yedges
    use advectz,    only : advectzzoom

    ! --- in/out -----------------------------------

    integer, intent(in)   ::  region
    type(TDate)           ::  tr(2)
    integer, intent(out)  ::  status

    ! --- const -------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/do_steps'
    
    ! --- local -----------------------------------
    
    integer          :: child, i123, ichild, tref_child, reg, rgi
    character        :: tobedone
    character(len=1) :: next_step, prev_step   !cmk
    integer          :: start_x, stop_x, start_y, stop_y, n !CMKTEMP

    ! --- begin -----------------------------------
    
    if ( okdebug .and. myid==root ) print *,'do_steps: region ',region

    !example (cmk)
    !   XYZ          EC CEX
    !      xyzeccezyx      cexyzzyxec
    ! in this case the operations e and c should be executed in 
    ! the interface region but not in the core of the zoom region 
    ! (otherwise double counting)
    ! This results in the most simple algorithm, because you should leave
    ! the DO_STEPS routine AFTER a Z OR after finishing all steps...

    ! note that the parent is responsible for the interface with the children
    ! this means that the interface may not be 'update' in case of (e.g.)
    ! xyz        a
    !    xyzaazyx
    ! in this case, the edges of the child are not updated with the proces a.
    ! this has consequences for the mmix and save output....
    ! THE INTERFACE IS PART OF THE PARENT, NOT THE CHILD.....

    do i123=1,n_operators  !WP! changed 3 to n_operators

      next_step = splitorderzoom(region,statusr(region)+1)
      if ( statusr(region) /= 0 ) then
        prev_step = splitorderzoom(region,statusr(region))
      else
        prev_step = ' '
      end if

      if ( okdebug .and. myid == root ) then

        ! it's only to make work of the code visible -- step to be done 
        ! will be printed with capital letter (X,Y or Z)
        do reg=1,region
          tobedone = ' '
          if ( reg == region ) tobedone = upper(next_step)
          print *, 'do_steps: ',reg,': ', &
               splitorderzoom(reg,1:statusr(reg)),tobedone
        end do

      end if
      
      select case(next_step)

        case ( 'v' )

          if ( convec_apply ) then

            !WP! data must be on tracers for convection 
            call Par_Check_domain(region,'n','tracer') 
            if ( okdebug ) call Par_Check_mass(region,'bef_convec')

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,0,'conv')
              call budget_transportg(region,0,'conv',prev_step)
            endif
#endif

            if ( okdebug ) print *, 'do_steps: Call convection'
            call convec(region)

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,1,'conv')
              call budget_transportg(region,1,'conv',prev_step)
            endif
#endif

            if ( okdebug ) call Par_Check_mass(region,'aft_convec')

          end if

        case ( 'c' )

          if ( chemistry_apply ) then

            !WP! data must be on levels for chemistry 
            call Par_Check_domain(region,'n','levels')
            if ( okdebug ) call Par_Check_mass(region,'bef_chem')

            if ( okdebug ) print*, 'do_steps: Call chemie'
            call Chemie( region, tr, status )
            IF_NOTOK_RETURN(status=1)

            if ( okdebug ) call Par_Check_mass(region,'aft_chem')

          end if

        case( 's' ) 

          if ( source_apply ) then

            !WP! data must be on tracers for convection 
            call Par_Check_domain(region,'n','tracer')
            if ( okdebug ) call Par_Check_mass(region,'bef_source')

            if ( okdebug ) print*, 'do_steps: Call source'
            call source1(region)
            call source2(region)

            if ( okdebug ) call Par_Check_mass(region,'aft_source')

          end if

        case( 'x' )

          if ( advect_apply ) then

            !WP! data must be on tracers for X
            call Par_Check_domain(region,'c','tracer')
            if ( okdebug ) call Par_Check_mass(region,'befX_tracers')

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,0,'advx')
              call budget_transportg(region,0,'advx',prev_step)
            endif
#endif

            call advectxzoom(region)

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,1,'advx')
              call budget_transportg(region,1,'advx',prev_step)
            endif
#endif

            if ( okdebug ) call Par_Check_mass(region,'aftX_tracers')
            
          end if

        case( 'y' )

          if ( advect_apply ) then

            !WP! data must be on tracers for Y
            call Par_Check_domain(region,'c','tracer')
            if ( okdebug ) call Par_Check_mass(region,'befY_tracers')

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,0,'advy')
              call budget_transportg(region,0,'advy',prev_step)
            endif
#endif
            
            call advectyzoom(region)

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,1,'advy')
              call budget_transportg(region,1,'advy',prev_step)
            endif
#endif

            if ( okdebug ) call Par_Check_mass(region,'aftY_tracers')

          end if

        case( 'z' )

          if ( advect_apply ) then

            !WP! data must be on tracers for Z
            call Par_Check_domain(region,'n','tracer')
            if ( okdebug ) call Par_Check_mass(region,'befZ_tracers')

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,0,'advz')
              call budget_transportg(region,0,'advz',prev_step)
            endif
#endif
            
            call advectzzoom(region)

#ifdef with_budgets
            if (apply_budget_global) then
              call budget_transport(region,1,'advz')
              call budget_transportg(region,1,'advz',prev_step)
            endif
#endif

            if ( okdebug ) call Par_Check_mass(region,'aftZ_tracers')
            
          end if

        case default

          print *,'do_steps:  strange value in splitorderzoom: ',  &
               splitorderzoom(region,statusr(region))
          print *,'do_steps:  (must be c, e, v, x, y or z)'
          call escape_tm('do_steps: Error') 

      end select

      statusr(region) = statusr(region)+1

      ! e.g. xyzvec...........cevzyx
      ! exit after xyz or vec of cevzyx....
      ! if the exit is after vec...the interface of child should be updated...
      ! 
      if ( mod(statusr(region),n_operators) == 0 ) then
        ! after xyz....vec...update child interface CMK removed feb 2003...
        ! BUT replaced in march 2003 by MK: budgets were corrupted when
        ! the edges are NOT updated!!!  still to find out why???
        if ( next_step /= 'x' ) then 
          call put_xedges(region)
          call put_yedges(region)
        end if    ! if not .... zyxcev
        exit ! e.g. vceecvzyx 
      end if

      !exit after 'yz' sequence to proces xyz children
      if ( next_step == 'z' ) then
         prev_step = splitorderzoom(region,statusr(region)-1)
         if ( prev_step == 'y' ) exit
      end if

    end do
    
    ! ok
    status = 0

  end subroutine do_steps

 
  ! ***
  

  character function upper(xyz)

    implicit none
    
    character(1),intent(in) :: xyz

    if (xyz=='x') then
       upper = 'X'
    else if (xyz=='y') then
       upper = 'Y'
    else if (xyz=='z') then
       upper = 'Z'
    else if (xyz=='c') then
       upper = 'C'
    else if (xyz=='s') then
       upper = 'S'
    else if (xyz=='v') then
       upper = 'V'
    else
       upper = '_'
    end if

  end function upper
 

end module modelIntegration
