!###############################################################################
!
!ProTeX: 1.14-AJS
!
!BOI
!
! !TITLE:        TMM - TM Meteo
! !AUTHORS:      Arjo Segers
! !AFFILIATION:  KNMI
! !DATE:         21/04/2004
!
! !INTRODUCTION: Introduction
!
!   Module to access TM meteo.
!
!   The main structure provides access to a list of opened meteo files.
!   If a new meteo field is required, the subroutines search in the
!   list wether the field is available. 
!   If not, a new file is opened and added to the list.
!   Optionally, a shell script is invoked to search for a file
!   and to store it locally if necessary.
!
!
! !INTRODUCTION: Usage
!
! \bv
!
!   ! --- modules -----------------------------------------
!
!   use GO, only : TDate, NewDate
!
!   use grid, only : TllGridInfo, Init, Done
!   use grid, only : TLevelInfo, Init, Done
!
!   use TMM, only : TTmMeteo
!   use TMM, only : Init, Done
!   use TMM, only : ReadField, ReadUVSP
!
!   ! --- local -----------------------------------------
!
!   type(TllGridInfo)           ::  lli
!   type(TLevelInfo)            ::  levi_ec, levi
!
!   type(TTmMeteo)              ::  tmmd
!
!   type(TDate)                 ::  tday, t1, t2
!
!   real                        ::  psurf(120,90)
!   real                        ::  temper(120,90,25)
!   real                        ::  pu(0:120,90,25)
!
!   ! --- begin -----------------------------------------
!
!   ! define horizontal grid
!   call Init( lli, -178.5, 3.0, 120, -89.0, 2.0, 90, status )
!
!   ! define vertical hybride levels
!   call Init( levi_ec, 'ec60', status )            ! ecmwf levels
!   call Init( levi, levi_ec, (/60,..,0/), status ) ! tm half level selection
!
!   ! setup TM meteo access:
!   call Init( tmmd, 'tm5.rc', status )
!
!   ! define time range for field:
!   tday = NewDate( year=2003, month=01, day=01 )
!   t1   = NewDate( year=2002, month=12, day=31, hour=21 )
!   t2   = NewDate( year=2003, month=01, day=01, hour=03 )
!
!   ! Read meteo arrays; specify grid, parameter, time, etc.
!   !
!   ! Type of grid is defined by nuv key:
!   !  'n' = normal grid (cell centers)
!   !  'u' = u-grid (east/west boundaries)
!   !  'v' = v-grid (north/south boundaries)
!   !
!   ! Requested grid (lli/levi) might be different from the grid in the file.
!   ! Horizontal:
!   !   o if file contains same resolutions as defined by lli,
!   !     a part of the data in the file is selected;
!   !   o if file contains higher resolution fields, 
!   !     the ll array is filled with combined values from the file;
!   !     depending on the parameter, the result is summed/avaraged/etc.
!   ! Vertical:
!   !   o if a file contains a supperset of the levels in levi,
!   !     some levels are combined;
!   !     depending on the parameter, the result is summed/avaraged/etc;
!   !   o the file might contain fields with reversed level order.
!   !
!   ! 3D fields require surface pressure for the level definition.
!   ! It should be valid for [t1,t2] !
!   ! Best is to use the spm from ReadUVSP.
!
!   call ReadUVSP ( tmmd, 'tmpp:od-fc-ml60-glb3x2', tday, t1, t2, lli, levi, sp1, spm, sp2, pu, pv, status )
!
!   call ReadField( tmmd, 'tmpp:od-fc-ml60-glb3x2', 'T' , tday, t1, t2, lli, 'n', levi, spm, temper, status )
!   call ReadField( tmmd, 'tmpp:od-fc-ml60-glb3x2', 'pu', tday, t1, t2, lli, 'u', levi, spm, pu    , status )
!
!   !
!   ! TMPP surface fields can be called using the 1x1 as well as the 3x2 key.
!   !
!   call ReadField( tmmd, 'tmpp:od-fc-sfc-glb1x1' , 'ci', tday, t1, t2, lli, 'n', ci, status )
!   call ReadField( tmmd, 'tmpp:od-fc-ml60-glb3x2', 'ci', tday, t1, t2, lli, 'n', ci, status )
!   !
!   ! Similar for spm surface pressures:
!   !
!   call ReadField( tmmd, 'tmpp:od-fc-ml1-glb3x2' , 'spm', tday, t1, t1, lli, 'n', spm, status )
!   call ReadField( tmmd, 'tmpp:od-fc-ml60-glb3x2', 'spm', tday, t1, t1, lli, 'n', spm, status )
!
!
!   ! *** output
!
!   call WriteField( tmmd, 'tmpp:od-fc-ml60-glb3x2', 'T', 'K', t1, t2, &
!                          lli, 'n', levi, spm, temper, status )
!
!   ! *** finish
!
!   call Done( tmmd, status )
!   call Done( levi, status    )
!   call Done( levi_ec, status )
!   call Done( lli )
!
!   ! -- end
!
! \ev
!
!
! !INTRODUCTION: Rcfile
!
! \bv
!
!   !
!   ! Meteo files are linked to or unpacked in a buffer directory.
!   !  o Set the clean flag (T|F) such that files that have not been accessed
!   !    for a long time are removed if a maximum buffer usage is exceeded.
!   !  o specify a maximum size in Mb
!   !
!   tmm.dir        : ${RUNDIR}/tmm-buf
!   tmm.dir.clean  : T
!   tmm.dir.size   : 500
!
!   !
!   ! TMM requires keys on how to form meteo for a certain region.
!   ! A key should be defined for each region, names are in 'dims_grid.F90'
!   ! For example:
!   !
!   !   tmpp:od-fc-ml60-glb3x2 
!   !     Read global 3x2, 60 level files produced by TMPP.
!   !     Optionally, the meteo is combined over levels or grid cells.
!   !     The files are expected to be present in the buffer directory
!   !     specified below after 'tmm.buf.dir' .
!   !     To have the appropriate files installed at the begin of a run,
!   !     use the 'tmm.setup.*' stuff below.
!   !
!   !   tmppS:od-fc-ml60-glb3x2 
!   !     Idem, but also calls a script to search for an appropriate file
!   !     from within the fortran program.
!   !     The system call to this script turned out to be rather slow.
!   !     This source type should be avoided therefore, but might be
!   !     very usefull in case of limitted disk space.
!   !
!   !   prism:
!   !     Receive meteo from the prism coupler.
!   !
!   tmm.sourcekey.glb6x4  : tmpp:od-fc-ml60-glb3x2
!   tmm.sourcekey.eur3x2  : tmpp:od-fc-ml60-glb3x2
!   tmm.sourcekey.eur1x1  : tmpp:od-fc-tropo25-eur1x1
!
!   !
!   ! Meteo files could be setup before the actual program is started.
!   ! Fill the following settings:
!   !  o set the apply flag apply this feature or not (T|F)
!   !  o specify a list of meteo files to be installed (spm,uvsp, etc)
!   !  o specify a list of meteo sources (od-fc-ml60-glb3x2 etc)
!   !  o specify wether message are printend or not (T|F)
!   !
!   tmm.setup.apply    :  T
!   tmm.setup.files    :  spm uvsp t q cld sub surf
!   tmm.setup.sources  :  od-fc-ml60-glb3x2
!   tmm.setup.verbose  :  T
!
!   !
!   ! Archive(s) to be searched for monthly tar files.
!   ! If more than one is specified (space seperated list), 
!   ! multiple directories are examined.
!   !  o disk archives
!   !  o tape archives ecfs/mos ('massive-storage-system')
!   !
!   tmm.search.disk   : ${DATADIR}/meteo
!   tmm.search.mss    : /nlh/TM/meteo
!
! \ev
!
!
! !INTRODUCTION: Source and scripts
!
! \bv
!
!   tmm.f90                : Main routines and collecting data structure.
!                            Provides access to a list of open meteo files.
!
!     tmm_mf.f90           : Search, open, close a meteo file.
!                            Calls shell script 'bin/tmm_getmeteo'.
!                            Calls specific routines for hdf/etc files.
!
!       tmm_mf_hdf.f90     : Read fields from hdf files.
!
!       tmm_param.f90      : Parameter specific stuff: 
!                            what to do with temperature fields, 
!                            what to do with mass fluxes etc.
!
! \ev
!
!EOI
!

!
! The spm/spmid problem ...
!
!   Glossary:
!     spm      : surface pressures for 00, 06, ...;
!                read from 'spm_' files, computed from ecmwf lnsp
!     spmid    : average of surface pressures at begin/end interval
!
!   To have the same algorithm as in TMPP, 
!   the following procedure should be used:
!
!    1) use ReadUVSP to get sp1, spm, sp2
!       spm is now in fact a spmid field (average of sp1 and sp2);
!       in future it should be the real spm:
!
!         call ReadUVSP( tmm, archivekey, tday, t1, t2, &
!                             lli, levi, sp1, spm, sp2, pu, pv, status )
!  
!    2) use this spm in calls to ReadField:
!
!         call ReadField( tmm, archivekey, paramkey, tday, t1, t2, &
!                              lli, nuv, levi, spm, ll, status )
!
!   Examples of current implementation:
!
!     1) Temperature 6x4x25 from 3x2x60
!          read spm 3x2 from file
!          horizontal mass average to 6x4x60 using spm 3x2 from
!          vertical combination to 6x4x25 using provided sp 3x2
!
!     2) Temperature 6x4x25 from N80x60
!          read spm N80
!          horizontal mass average to 6x4x60 using spm N80 and provided sp 3x2
!            --> should be spm 3x2 from spm N80
!
!###############################################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!###############################################################################

module TMM

  use GO      , only : TDate
  use Grid    , only : TllGridInfo, TggGridInfo, TshGridInfo, TLevelInfo
  use tmm_mf  , only : TMeteoFile
  use tmm_info, only : TMeteoInfo, SetHistory, AddHistory

#include "tmm.h"

  implicit none
  
  ! --- in/out -------------------------------------
  
  private
  
  public  ::  TTmMeteo
  public  ::  Init, Done

  public  ::  ReadField
  public  ::  ReadUVSP, Read_SP, Read_MFUV, Read_MFW, Read_TQ
  public  ::  Read_Convec, Read_CloudCovers
  public  ::  Read_SR_OLS
!  public  ::  ReadEqvLat
  
  public  ::  WriteField
  
  public  ::  TMeteoInfo, SetHistory, AddHistory
  
  ! --- const ---------------------------------------
  
  character(len=*), parameter  ::  mname = 'tmm'
  
  ! maximum number of opened meteo files
  integer, parameter           ::  maxmf = 60
  
  
  ! --- types ---------------------------------------
  
  type TTmMeteo
    ! rc file with paths etc
    character(len=256)            ::  rcfilename
    ! paths
    character(len=256)            ::  input_dir, output_dir
    ! list of meteo files
    integer                       ::  nmf
    type(TMeteoFile)              ::  mf(maxmf)
    !
    ! buffers for latest read raw field
    !
    logical                    ::  buf_filled
    !
    character(len=10)          ::  buf_archivekey
    character(len=10)          ::  buf_paramkey
    type(TDate)                ::  buf_tday, buf_t1, buf_t2
    character(len=1)           ::  buf_nuv
    character(len=1)           ::  buf_nw
    type(TMeteoInfo)           ::  buf_tmi
    !
    character(len=2)           ::  buf_gridtype
    !
    type(TllGridInfo)          ::  buf_lli
    real, pointer              ::  buf_ll(:,:,:)
    real, pointer              ::  buf_sp_ll(:,:)
    !
    type(TggGridInfo)          ::  buf_ggi
    real, pointer              ::  buf_gg(:,:)
    real, pointer              ::  buf_sp_gg(:)
    !
    type(TshGridInfo)          ::  buf_shi
    complex, pointer           ::  buf_sh(:,:)
    complex, pointer           ::  buf_lnsp_sh(:)
    !
    type(TLevelInfo)           ::  buf_levi
    !
    ! storage for horizontal interpol
    real, pointer              ::  llX(:,:,:)
  end type TTmMeteo  


  ! --- interfaces -------------------------------------
  
  interface Init
    module procedure tmm_Init
  end interface

  interface Done
    module procedure tmm_Done
  end interface

  interface SelectMF
    module procedure tmm_SelectMF
  end interface

  interface ReadField
    module procedure tmm_ReadField_2d
    module procedure tmm_ReadField_3d
  end interface

  interface ReadUVSP
    module procedure tmm_ReadUVSP
  end interface

  interface Read_SP
    module procedure tmm_Read_SP
  end interface

  interface Read_MFUV
    module procedure tmm_Read_MFUV
  end interface

  interface Read_MFW
    module procedure tmm_Read_MFW
  end interface

  interface Read_TQ
    module procedure tmm_Read_TQ
  end interface

  interface Read_Convec
    module procedure tmm_Read_Convec
  end interface

  interface Read_CloudCovers
    module procedure tmm_Read_CloudCovers
  end interface

  interface Read_SR_OLS
    module procedure tmm_Read_SR_OLS
  end interface

!  interface ReadEqvLat
!    module procedure tmm_ReadEqvLat
!  end interface

  interface WriteField
    module procedure tmm_WriteField_2d
    module procedure tmm_WriteField_3d
  end interface


contains


  ! ===================================================================
  
  
  subroutine tmm_Init( tmm, rcfilename, status )

    use PArray, only : pa_Init
    use GO    , only : goErr, gol, goPr
    use GO, only : NewDate
    use GO, only : TrcFile, Init, Done, ReadRc
    
#ifdef pgf90
    use parray_r4
    use parray_r8
    use parray_c4
    use parray_c8
#endif
  
    ! --- in/out ----------------------------------
    
    type(TTmMeteo), intent(out)         ::  tmm
    character(len=*), intent(in)        ::  rcfilename
    integer, intent(out)                ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Init'
    
    ! --- local -----------------------------------
    
    type(TrcFile)     ::  rcF
    
    ! --- begin -----------------------------------
    
    ! store rc file name
    tmm%rcfilename = rcfilename
    
    ! read paths
    call Init( rcF, rcfilename, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'tmm.dir', tmm%input_dir, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'tmm.output.dir', tmm%output_dir, status )
    IF_NOTOK_RETURN(status=1)
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    
    ! no files open yet
    tmm%nmf = 0

    ! buffer empty
    tmm%buf_filled     = .false.
    tmm%buf_archivekey = 'none'
    tmm%buf_paramkey   = 'none'
    tmm%buf_tday       = NewDate(time5=(/0001,01,01,00,00/))
    tmm%buf_t1         = NewDate(time5=(/0001,01,01,00,00/))
    tmm%buf_t2         = NewDate(time5=(/9999,12,31,00,00/))
    tmm%buf_nuv        = 'n'
    tmm%buf_nw         = 'n'
    tmm%buf_gridtype   = 'xx'
    call pa_Init( tmm%buf_ll )
    call pa_Init( tmm%buf_sp_ll )
    call pa_Init( tmm%buf_gg )
    call pa_Init( tmm%buf_sp_gg )
    call pa_Init( tmm%buf_sh )
    call pa_Init( tmm%buf_lnsp_sh )
    
    ! init temp grid on destination grid but raw levels
    call pa_Init( tmm%llX )
    
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    
    ! ok
    status = 0

  end subroutine tmm_Init
  
  
  ! ***
  
  
  subroutine tmm_Done( tmm, status )
  
    use PArray, only : pa_Done
    use GO    , only : goErr, gol, goPr
    use tmm_mf, only : Opened, Done
    
#ifdef pgf90
    use parray_r4
    use parray_r8
    use parray_c4
    use parray_c8
#endif
  
    ! --- in/out ----------------------------------
    
    type(TTmMeteo), intent(inout)          ::  tmm
    integer, intent(out)                   ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Done'
    
    ! --- local ----------------------------------------
    
    integer          ::  imf
    
    ! --- begin -------------------------------------------

    ! loop over all available meteo files
    do imf = 1, tmm%nmf
    
      ! in use ?
      if ( .not. Opened( tmm%mf(imf) ) ) cycle

      ! close ...
      call Done( tmm%mf(imf), status )
      IF_NOTOK_RETURN(status=1)
      
    end do
    
    ! clear buffer
    call ClearBuffer( tmm, status )
    IF_NOTOK_RETURN(status=1)
    call pa_Done( tmm%buf_ll )
    call pa_Done( tmm%buf_sp_ll )
    call pa_Done( tmm%buf_gg )
    call pa_Done( tmm%buf_sp_gg )
    call pa_Done( tmm%buf_sh )
    call pa_Done( tmm%buf_lnsp_sh )
    
    ! clear temp grid
    call pa_Done( tmm%llX )
    
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    
    ! ok
    status = 0
    
  end subroutine tmm_Done
  

  ! ***
  
  
  ! Select meteo file for this param and time,
  ! search for a new file if necessary
  
  subroutine tmm_SelectMF( tmm, io, archivekey, paramkey, tday, t1, t2, imf, status )
 
    use GO    , only : goErr, gol, goPr
    use GO    , only : TDate
    use tmm_mf, only : Init, Done, Opened, CheckTime, CheckParam, SetupInput
    use tmm_mf, only : SetupInput, SetupOutput
    
    ! --- in/out ----------------------------------------
    
    type(TTmMeteo), intent(inout)           ::  tmm
    character(len=1), intent(in)            ::  io
    character(len=*), intent(in)            ::  archivekey
    character(len=*), intent(in)            ::  paramkey
    type(TDate), intent(in)                 ::  tday, t1, t2
    integer, intent(out)                    ::  imf
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_SelectMF'
    
    ! --- local ----------------------------------------
    
    integer                  ::  i
    
    ! --- begin ----------------------------------------
    
    ! not found yet ...
    imf = -1
    
    ! loop over currently used meteo files
    do i = 1, tmm%nmf
      ! in use ?
      if ( .not. Opened( tmm%mf(i) ) ) cycle
      ! test on requested grid and param:
      call CheckParam( tmm%mf(i), io, archivekey, paramkey, status )
      if ( status == 0 ) then
        ! param included: leave
        imf = i
        exit
      else if ( status < 0 ) then
        ! param not included, try next
        cycle
      else
        ! error ...
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      end if
    end do

    ! time ok ? close if data in file is too old:
    if ( imf > 0 ) then
      ! time ok ?
      call CheckTime( tmm%mf(imf), t1, t2, status )
      if ( status == 0 ) then
        ! mf includes [t1,t2]; return with this imf
        status = 0; return
      else if ( status < 0  ) then 
        ! mf does not include [t1,t2]; close file
        call Done( tmm%mf(imf), status )
        IF_NOTOK_RETURN(status=1)
        ! not in use anymore ...
        imf = -1
      else
        call goErr; status = 1; return
      end if
    end if

    ! open new meteo file ?
    if ( imf < 0 ) then
      ! search first available empty mf
      do i = 1, tmm%nmf
        if ( .not. Opened(tmm%mf(i)) ) then
          imf = i
          exit
        end if
      end do
      ! next number ?
      if ( imf < 0 ) then
        tmm%nmf = tmm%nmf + 1
        if ( tmm%nmf > maxmf ) then
          write (gol,'("Tried to init meteo file beyond maximum number: ",i6)') maxmf; call goErr
          write (gol,'("Initialized files:")'); call goErr
          do i = 1, maxmf
            write (gol,'("  ",a)') trim(tmm%mf(i)%filename); call goErr
          end do
          write (gol,'("Please increase parameter `maxmf` in ",a)') mname; call goErr
          call goErr; status = 1; return
        end if
        imf = tmm%nmf
      end if
      ! start new mf ...
      call Init( tmm%mf(imf), io, status )
      IF_NOTOK_RETURN(status=1)
    end if
    
    ! input or output ?
    select case ( io )

      case ( 'i' )  ! input

        ! open file, store description, etc
        call SetupInput( tmm%mf(imf), archivekey, paramkey, tday, t1, t2, &
                                      tmm%rcfilename, tmm%input_dir, status )
        IF_NOTOK_RETURN(status=1)

      case ( 'o' )  ! output

        ! open file, store description, etc
        call SetupOutput( tmm%mf(imf), archivekey, paramkey, tday, t1, t2, &
                                       tmm%rcfilename, tmm%output_dir, status )
        IF_NOTOK_RETURN(status=1)

      case default
        write (gol,'("unsupported io `",a,"`")') io; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
    end select

    ! ok
    status = 0    
    
  end subroutine tmm_SelectMF
    

  ! ==================================================================
  ! ===
  ! ===  buffer
  ! ===
  ! ==================================================================
  
  
  subroutine FillBuffer( tmm, archivekey, paramkey, tday, t1, t2, nuv, nw, status )
  
    use GO    , only : goErr, gol, goPr
    use GO    , only : TDate, operator(==)
    use tmm_mf, only : ReadRecord

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    character(len=*), intent(in)            ::  archivekey
    character(len=*), intent(in)            ::  paramkey
    type(TDate), intent(in)                 ::  tday, t1, t2
    character(len=1), intent(in)            ::  nuv
    character(len=1), intent(in)            ::  nw
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/FillBuffer'
    
    ! --- local -------------------------------
    
    integer          ::  imf

    ! --- begin -------------------------------
    
    if ( (archivekey == tmm%buf_archivekey) .and. (paramkey == tmm%buf_paramkey) .and. &
         (nuv == tmm%buf_nuv) .and. (nw == tmm%buf_nw) .and. &
         (tday == tmm%buf_tday) .and. (t1 == tmm%buf_t1) .and. (t2 == tmm%buf_t2) ) then
      ! requested field already in buffer ...
      ! ok
      status = 0; return
    end if

    ! select (eventually retrieve first) the meteo file with this param:
    call SelectMF( tmm, 'i', archivekey, paramkey, tday, t1, t2, imf, status )
    IF_NOTOK_RETURN(status=1)
    
    ! clear buffer
    call ClearBuffer( tmm, status )
    IF_NOTOK_RETURN(status=1)
    
    ! fill keys:
    tmm%buf_archivekey = archivekey
    tmm%buf_paramkey   = paramkey
    tmm%buf_t1         = t1
    tmm%buf_t2         = t2
    tmm%buf_tday       = tday
    tmm%buf_nuv        = nuv
    tmm%buf_nw         = nw

    ! read field:
    call ReadRecord( tmm%mf(imf), paramkey, t1, t2, nuv, nw, &
                     tmm%buf_gridtype, tmm%buf_levi, &
                     tmm%buf_lli, tmm%buf_ll, tmm%buf_sp_ll, &
                     tmm%buf_ggi, tmm%buf_gg, tmm%buf_sp_gg, &
                     tmm%buf_shi, tmm%buf_sh, tmm%buf_lnsp_sh, &
                     tmm%buf_tmi, status )
    IF_NOTOK_RETURN(status=1)

    ! some data present ..    
    tmm%buf_filled   = .true.

    ! ok
    status = 0
    
  end subroutine FillBuffer
  

  ! ***
  
    
  subroutine ClearBuffer( tmm, status )

    use GO      , only : goErr, gol, goPr
    use PArray  , only : pa_Done  
    use GO      , only : goErr
    use Grid    , only : Done
    use tmm_info, only : Done
  
#ifdef pgf90
    use parray_r4
    use parray_r8
    use parray_c4
    use parray_c8
#endif
  
    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/ClearBuffer'
    
    ! --- begin -------------------------------
    
    if ( tmm%buf_filled ) then
    
      call Done( tmm%buf_tmi, status )
      IF_NOTOK_RETURN(status=1)

      if ( tmm%buf_gridtype == 'll' ) then
        call Done( tmm%buf_lli, status )
        IF_NOTOK_RETURN(status=1)
        call pa_Done( tmm%buf_ll    )
        call pa_Done( tmm%buf_sp_ll )
      end if

      if ( tmm%buf_gridtype == 'gg' ) then
        call Done( tmm%buf_ggi, status )
        IF_NOTOK_RETURN(status=1)
        call pa_Done( tmm%buf_gg    )
        call pa_Done( tmm%buf_sp_gg )
      end if

      if ( tmm%buf_gridtype == 'sh' ) then
        call Done( tmm%buf_shi )
        call pa_Done( tmm%buf_sh    )
        call pa_Done( tmm%buf_lnsp_sh )
      end if

      call Done( tmm%buf_levi, status )
      IF_NOTOK_RETURN(status=1)
      
    end if
    
    ! ok
    status = 0
    
  end subroutine ClearBuffer
  

  ! ***
  
    
  subroutine CheckBuffer( tmm, status, gridtype, np, shT, nlev )

    use GO, only : goErr, gol, goPr

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    integer, intent(out)                    ::  status
    character(len=*), intent(in), optional  ::  gridtype
    integer, intent(in), optional           ::  np
    integer, intent(in), optional           ::  shT
    integer, intent(in), optional           ::  nlev
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/CheckBuffer'
    
    ! --- begin -------------------------------
    
    if ( .not. tmm%buf_filled ) then
      write (gol,'("buffer not filled ...")'); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if
    
    if ( present(gridtype) ) then
      if ( tmm%buf_gridtype /= gridtype ) then
        write (gol,'("unexpected gridtype in buffer:")'); call goErr
        write (gol,'("  buffer     : ",a)') tmm%buf_gridtype; call goErr
        write (gol,'("  expected   : ",a)') gridtype; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      end if
    end if
    
    if ( present(nlev) ) then
      if ( tmm%buf_levi%nlev /= nlev ) then
        write (gol,'("unexpected number of levels in buffer:")'); call goErr
        write (gol,'("  buffer     : ",i4)') tmm%buf_levi%nlev ; call goErr
        write (gol,'("  expected   : ",i4)') nlev ; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      end if
    end if

    if ( present(np) ) then
      if ( tmm%buf_ggi%np /= np ) then
        write (gol,'("unexpected grid size in buffer:")'); call goErr
        write (gol,'("  buffer ggi%np : ",i6)') tmm%buf_ggi%np; call goErr
        write (gol,'("  expected      : ",i6)') np; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      end if
    end if
    
    if ( present(shT) ) then
      if ( tmm%buf_shi%T /= shT ) then
        write (gol,'("unexpected grid size in buffer:")'); call goErr
        write (gol,'("  buffer shi%shT : ",i6)') tmm%buf_shi%T; call goErr
        write (gol,'("  expected       : ",i6)') shT; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      end if
    end if
    
    ! ok
    status = 0

  end subroutine CheckBuffer
  
  
  ! ***
  
  
  subroutine Transform2D( tmm, lli, nuv, tmi, status )

    use PArray       , only : pa_SetShape
    use GO           , only : goErr, gol, goPr
    use Grid         , only : TllGridInfo, FillGrid
    use Grid         , only : Tgg2llFracs, Init, Done
    use Grid         , only : Interpol
#ifdef MIPSpro
    use Grid_Interpol, only : FracSum
#else
    use Grid         , only : FracSum
#endif
    use tmm_param    , only : GetCombineKeys
    use tmm_info     , only : TMeteoInfo, Init, AddHistory
    
#ifdef pgf90
    use parray_r4
    use parray_r8
    use parray_c4
    use parray_c8
#endif
  
    ! --- in/out --------------------------------
    
    type(TTmMeteo), intent(inout)           ::  tmm
    type(TllGridInfo), intent(in)           ::  lli
    character(len=1), intent(in)            ::  nuv
    type(TMeteoInfo), intent(out)           ::  tmi
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/Transform2D'
    
    ! --- local -------------------------------
    
    character(len=10)       ::  hcomb, vcomb
    type(Tgg2llFracs)       ::  gg2ll
    
    ! --- begin ----------------------------------
    
    ! copy info:
    tmi = tmm%buf_tmi

    ! set shape of target grid:
    select case ( nuv )
      case ( 'n' )
        call pa_SetShape( tmm%llX, lli%nlon  , lli%nlat  , 1 )
      case ( 'u' )
        call pa_SetShape( tmm%llX, lli%nlon+1, lli%nlat  , 1 )
      case ( 'v' )
        call pa_SetShape( tmm%llX, lli%nlon  , lli%nlat+1, 1 )
      case default
        write (gol,'("unsupported nuv `",a,"`")') nuv
        write (gol,'("in ",a)') rname; call goErr; status=1; return
    end select
    
    ! fill with zero's for safety:
    tmm%llX = 0.0
    
    ! define how to combine horizontal cells and vertical levels:
    call GetCombineKeys( hcomb, vcomb, tmm%buf_paramkey, status )
    IF_NOTOK_RETURN(status=1)
    
    ! transform raw field to ll :
    select case ( tmm%buf_gridtype )
    
      ! ~~~ lat/lon ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'll' )

        ! copy or combine horizontal cells from buffer into lli/llX :
        call FillGrid( lli, nuv, tmm%llX(:,:,1), &
                       tmm%buf_lli, tmm%buf_nuv, tmm%buf_ll(:,:,1), &
                       hcomb, status )
        IF_NOTOK_RETURN(status=1)
        
      ! ~~~ gg ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'gg' )
      
        ! determine fraction
        call Init( gg2ll, tmm%buf_ggi, lli, status )
        IF_NOTOK_RETURN(status=1)

        ! deceide how to interpolate given hcomb key:
        select case ( hcomb )

          case ( 'area-aver' )
          
            ! take fractions of overlapping cells
            call FracSum( gg2ll, tmm%buf_gg(:,1), tmm%llX(:,:,1), status, 'area-aver' )
            IF_NOTOK_RETURN(status=1)
            
          case default

            write (gol,'("unsupported horizonal combination key for gg :",a)') hcomb; call goErr
            write (gol,'("in ",a)') rname; call goErr; status=1; return
      
        end select
            
        ! done
        call Done( gg2ll )

      ! ~~~ sh ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'sh' )
      
        ! interpolate field in shi/sh into lli/ll :
        call Interpol( tmm%buf_shi, tmm%buf_sh(:,1), lli, tmm%llX(:,:,1), status )
        IF_NOTOK_RETURN(status=1)
        
      case default

        write (gol,'("unsupported input grid type `",a,"`")') tmm%buf_gridtype; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      
    end select
    
    ! fill history:
    call Init( tmi, tmm%buf_tmi, status )
    call AddHistory( tmi, 'oper==hcomb,'//trim(hcomb), status )

    ! ok
    status = 0

  end subroutine Transform2D


  ! ***
  
  
  subroutine Transform3Dh( tmm, lli, nuv, levi, nw, sp, tmi, status )

    use PArray       , only : pa_SetShape
    use GO           , only : goErr, gol, goPr
    use Grid         , only : TLevelInfo, FillLevels
    use Grid         , only : TllGridInfo, FillGrid, AreaOper
    use Grid         , only : Tgg2llFracs, Init, Done
    use Grid         , only : IntArea
#ifdef MIPSpro
    use Grid_Interpol, only : FracSum
#else
    use Grid         , only : FracSum
#endif
    use tmm_param    , only : GetCombineKeys
    use tmm_info     , only : TMeteoInfo, Init, AddHistory
    
#ifdef pgf90
    use parray_r4
    use parray_r8
    use parray_c4
    use parray_c8
#endif
  
    ! --- in/out --------------------------------
    
    type(TTmMeteo), intent(inout)           ::  tmm
    type(TllGridInfo), intent(in)           ::  lli
    character(len=1), intent(in)            ::  nuv
    type(TLevelInfo), intent(in)            ::  levi
    character(len=1), intent(in)            ::  nw
    real, intent(out)                       ::  sp(:,:) 
    type(TMeteoInfo), intent(out)           ::  tmi
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/Transform3Dh'
    
    ! --- local -------------------------------
    
    integer                 ::  nlon, nlat, nlev
    character(len=10)       ::  hcomb, vcomb
    integer                 ::  l
    type(Tgg2llFracs)       ::  gg2ll
    
    real, allocatable       ::  dp_ll(:,:)
    real, allocatable       ::  dp_llX(:,:)
    real, allocatable       ::  dp_gg(:)
    
    ! --- begin ----------------------------------
    
    ! set shape of target grid:
    select case ( nuv )
      case ( 'n' )
        nlon = lli%nlon
        nlat = lli%nlat
      case ( 'u' )
        nlon = lli%nlon+1
        nlat = lli%nlat
      case ( 'v' )
        nlon = lli%nlon
        nlat = lli%nlat+1
      case default
        write (gol,'("unsupported nuv `",a,"`")') nuv; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
    end select
    select case ( nw )
      case ( 'n' )
        nlev = tmm%buf_levi%nlev
      case ( 'w' )
        nlev = tmm%buf_levi%nlev+1
      case default
        write (gol,'("unsupported nw `",a,"`")') nw; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
    end select
    call pa_SetShape( tmm%llX, nlon, nlat, nlev )
    tmm%llX = 0.0
    
    ! define how to combine horizontal cells and vertical levels:
    call GetCombineKeys( hcomb, vcomb, tmm%buf_paramkey, status )
    IF_NOTOK_RETURN(status=1)

    ! transform raw field to ll :
    select case ( tmm%buf_gridtype )
    
      ! ~~~ lat/lon ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'll' )
      
        !! check size of buffer array
        !if ( size(tmm%buf_ll,3) /= nlev ) then
        !  write (gol,'("buffer array does not match with level definition:")')
        !  write (gol,'("  nw             : ",a )') nw
        !  write (gol,'("  buf_levi nlev  : ",i3)') tmm%buf_levi%nlev
        !  write (gol,'("  buf_ll levels  : ",i3)') size(tmm%buf_ll,3)
        !  write (gol,'("in ",a)') rname; call goErr; status=1; return
        !end if
        ! convective fluxes have only lowest layers ...
        if ( size(tmm%buf_ll,3) > nlev ) then
          write (gol,'("buffer array has more layers than implied by nw and level definition:")'); call goErr
          write (gol,'("  nw             : ",a )') nw; call goErr
          write (gol,'("  buf_levi nlev  : ",i3)') tmm%buf_levi%nlev; call goErr
          write (gol,'("  buf_ll levels  : ",i3)') size(tmm%buf_ll,3); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! surface pressure: aver horizontal cells from buffer into lli/llX :
        call FillGrid( lli, nuv, sp, &
                       tmm%buf_lli, tmm%buf_nuv, tmm%buf_sp_ll, &
                       'area-aver', status )
        IF_NOTOK_RETURN(status=1)
        
        ! deceide how to interpolate given hcomb key;
        ! for most keys, let 'FillGrid' determine what to do ...
        select case ( hcomb )

          case ( 'mass-aver' )
          
            ! check ...
            if ( nw /= 'n' ) then
              write (gol,'("nw should be `n` for mass aver ...")'); call goErr
              write (gol,'("in ",a)') rname; call goErr; status=1; return
            end if
            
            !
            ! mass weighted fractions:
            !
            !    sum_i  f_i dp_i/g dA_i
            !    ----------------------
            !      sum_i  dp_i/g dA_i
            !
      
            ! temporary pressure field:
            allocate( dp_ll(tmm%buf_lli%nlon,tmm%buf_lli%nlat) )
          
            ! loop over layers
            !do l = 1, nlev
            do l = 1, size(tmm%buf_ll,3)
              ! pressure gradients in this layer:
              dp_ll  = tmm%buf_levi%da(l) + tmm%buf_levi%db(l) * tmm%buf_sp_ll
              call AreaOper( tmm%buf_lli, dp_ll, '*', 'm2', status ) 
              IF_NOTOK_RETURN(status=1)
              ! copy or combine horizontal cells from buffer into lli/llX;
              ! combining cells is weighted by 'mass' dp_ll :
              call FillGrid( lli, nuv, tmm%llX(:,:,l), &
                             tmm%buf_lli, tmm%buf_nuv, tmm%buf_ll(:,:,l), &
                             'weight', status, dp_ll )
              IF_NOTOK_RETURN(status=1)
            end do
          
            ! clear:            
            deallocate( dp_ll )
            
          case default

            ! loop over layers in ll array:
            !do l = 1, nlev
            do l = 1, size(tmm%buf_ll,3)
              ! copy or combine horizontal cells from buffer into lli/llX;
              ! pass hcomb to FillGrid to define how cells are combined:
              call FillGrid( lli, nuv, tmm%llX(:,:,l), &
                             tmm%buf_lli, tmm%buf_nuv, tmm%buf_ll(:,:,l), &
                             hcomb, status )
              IF_NOTOK_RETURN(status=1)
            end do

        end select

      ! ~~~ gg ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'gg' )
      
        ! check size of buffer array
        if ( size(tmm%buf_gg,2) /= nlev ) then
          write (gol,'("buffer array does not match with level definition:")'); call goErr
          write (gol,'("  nw             : ",a )') nw; call goErr
          write (gol,'("  buf_levi nlev  : ",i3)') tmm%buf_levi%nlev; call goErr
          write (gol,'("  buf_gg levels  : ",i3)') size(tmm%buf_gg,2); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! determine fraction
        call Init( gg2ll, tmm%buf_ggi, lli, status )
        IF_NOTOK_RETURN(status=1)

        ! surface pressure: take fractions of overlapping cells
        call FracSum( gg2ll, tmm%buf_sp_gg, sp, status, 'area-aver' )
        IF_NOTOK_RETURN(status=1)
            
        ! deceide how to interpolate given hcomb key:
        select case ( hcomb )

          case ( 'area-aver' )
          
            !
            ! area weighted fractions:
            !
            !    sum_i  f_i dA_i
            !    ---------------
            !      sum_i  dA_i
            !
      
            ! loop over layers
            do l = 1, nlev
              ! take fractions of overlapping cells
              call FracSum( gg2ll, tmm%buf_gg(:,l), tmm%llX(:,:,l), status, 'area-aver' )
              IF_NOTOK_RETURN(status=1)
            end do
            
          case ( 'mass-aver' )
          
            ! check ...
            if ( nw /= 'n' ) then
              write (gol,'("nw should be `n` for mass aver ...")'); call goErr
              write (gol,'("in ",a)') rname; call goErr; status=1; return
            end if
            
            !
            ! mass weighted fractions:
            !
            !    sum_i  f_i dp_i/g dA_i
            !    ----------------------
            !      sum_i  dp_i/g dA_i
            !
      
            ! temporary pressure field:
            allocate( dp_gg(size(tmm%buf_sp_gg)) )
            allocate( dp_llX(size(sp,1),size(sp,2)) )
            
            ! loop over layers
            do l = 1, nlev
              ! pressure gradients in this layer:
              dp_gg  = tmm%buf_levi%da(l) + tmm%buf_levi%db(l) * tmm%buf_sp_gg
              dp_llX = tmm%buf_levi%da(l) + tmm%buf_levi%db(l) * sp
              ! take fractions of overlapping cells:
              !   sum_i  f_i  dp_i  dA_i
              call FracSum( gg2ll, tmm%buf_gg(:,l)*dp_gg, tmm%llX(:,:,l), status, 'area-sum' )
              IF_NOTOK_RETURN(status=1)
              ! weight by total dp dA
              tmm%llX(:,:,l) = tmm%llX(:,:,l) / dp_llX
              call AreaOper( lli, tmm%llX(:,:,l), '/', 'm2', status )
              IF_NOTOK_RETURN(status=1)
            end do

            ! clear:            
            deallocate( dp_gg )
            deallocate( dp_llX )
            
          case default

            write (gol,'("unsupported horizonal combination key for gg :",a)') hcomb; call goErr
            write (gol,'("in ",a)') rname; call goErr; status=1; return
      
        end select
            
        ! done
        call Done( gg2ll )

      ! ~~~ sh ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'sh' )
      
        ! check size of buffer array
        if ( size(tmm%buf_sh,2) /= nlev ) then
          write (gol,'("buffer array does not match with level definition:")'); call goErr
          write (gol,'("  nw             : ",a )') nw; call goErr
          write (gol,'("  buf_levi nlev  : ",i3)') tmm%buf_levi%nlev; call goErr
          write (gol,'("  buf_sh levels  : ",i3)') size(tmm%buf_sh,2); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! surface pressure: interpolate lnsp to target horizontal resolution:
        call IntArea( 'exp,aver', tmm%buf_shi, tmm%buf_lnsp_sh, lli, sp, status )
        IF_NOTOK_RETURN(status=1)
        
        ! deceide how to interpolate given hcomb key:
        select case ( hcomb )

          !case ( 'area-aver' )
          !
          !  ! area average over sepectral fields:
          !  call IntArea( 'aver', tmm%buf_shi, tmm%buf_sh, lli, tmm%llX, status )
          !  IF_NOTOK_RETURN(status=1)

          case ( 'mass-aver' )

            ! check ...
            if ( nw /= 'n' ) then
              write (gol,'("nw should be `n` for mass aver ...")'); call goErr
              write (gol,'("in ",a)') rname; call goErr; status=1; return
            end if
            
            ! mass average over sepectral fields:
            call IntArea( '[F*(da+db*exp(H))*cos]/[()*cos]', &
                          tmm%buf_shi, tmm%buf_sh, &
                          tmm%buf_lnsp_sh, tmm%buf_levi%da, tmm%buf_levi%db, &
                          lli, tmm%llX, status )
            IF_NOTOK_RETURN(status=1)

          case default

            write (gol,'("unsupported horizonal combination key for sh :",a)') hcomb; call goErr
            write (gol,'("in ",a)') rname; call goErr; status=1; return
      
        end select
            
      ! ~~~ ?? ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case default

        write (gol,'("unsupported input grid type `",a,"`")') tmm%buf_gridtype; call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      
    end select
    
    ! fill history:
    call Init( tmi, tmm%buf_tmi, status )
    call AddHistory( tmi, 'oper==hcomb,'//trim(hcomb), status )

    ! ok
    status = 0
  
  end subroutine Transform3Dh


  ! ***
  

  subroutine Transform3Dv( tmm, levi, nw, sp, ll, tmi, status )

    use PArray       , only : pa_SetShape
    use GO           , only : goErr, gol, goPr
    use Grid         , only : TLevelInfo, FillLevels
    use Grid         , only : TllGridInfo, FillGrid, AreaOper
    use Grid         , only : Tgg2llFracs, Init, Done
    use Grid         , only : IntArea
#ifdef MIPSpro
    use Grid_Interpol, only : FracSum
#else
    use Grid         , only : FracSum
#endif
    use tmm_param    , only : GetCombineKeys
    use tmm_info     , only : TMeteoInfo, AddHistory
    
    ! --- in/out --------------------------------
    
    type(TTmMeteo), intent(inout)           ::  tmm
    type(TLevelInfo), intent(in)            ::  levi
    character(len=1), intent(in)            ::  nw
    real, intent(in)                        ::  sp(:,:) 
    real, intent(out)                       ::  ll(:,:,:) 
    type(TMeteoInfo), intent(inout)         ::  tmi
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/Transform3Dv'
    
    ! --- local -------------------------------
    
    character(len=10)       ::  hcomb, vcomb
    
    ! --- begin ----------------------------------
    
    ! define how to combine horizontal cells and vertical levels:
    call GetCombineKeys( hcomb, vcomb, tmm%buf_paramkey, status )
    IF_NOTOK_RETURN(status=1)

    ! collect or copy levels, eventually reversed, from leviX/llX into levi/ll :
    !write (gol,'(a,": vertical ...")') rname
    call FillLevels( levi, nw, sp, ll, tmm%buf_levi, tmm%llX, vcomb, status )
    IF_NOTOK_RETURN(status=1)
    
    ! add history:
    call AddHistory( tmi, 'oper==vcomb,'//trim(vcomb), status )

    ! ok
    status = 0
  
  end subroutine Transform3Dv


  ! ==================================================================
  ! ===
  ! ===  read ll field
  ! ===
  ! ==================================================================


  subroutine tmm_ReadField_2d( tmm, archivekey, paramkey, tday, t1, t2, &
                                    lli, nuv, ll, tmi, status )
  
    use PArray  , only : pa_Init, pa_Done
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol
    use Grid    , only : TllGridInfo
    use tmm_info, only : TMeteoInfo

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    character(len=*), intent(in)            ::  archivekey
    character(len=*), intent(in)            ::  paramkey
    type(TDate), intent(in)                 ::  tday, t1, t2
    type(TllGridInfo), intent(in)           ::  lli
    character(len=1), intent(in)            ::  nuv
    real, intent(out)                       ::  ll(:,:)
    type(TMeteoInfo), intent(out)           ::  tmi
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_ReadField_2d'
    
    ! --- local -------------------------------
    
    character(len=10)       ::  hcomb, vcomb

    ! --- begin ----------------------------------
    
    call wrtgol( 'tmm read  : '//trim(paramkey)//' ', t1, '  -  ', t2 ); call goPr

    ! check shape of target grid:
    if ( ((nuv == 'n') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'u') .and. ((size(ll,1) /= lli%nlon+1) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'v') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat+1))) ) then
      write (gol,'("target array does not match with grid definition:")'); call goErr
      write (gol,'("  param  : ",a          )') paramkey; call goErr
      write (gol,'("  lli    : ",i3," x ",i3)') lli%nlon, lli%nlat; call goErr
      write (gol,'("  nuv    : ",a          )') nuv; call goErr
      write (gol,'("  ll     : ",i3," x ",i3)') shape(ll); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if
    
    !
    ! convert grid
    !

    ! read raw field in buffer
    call FillBuffer( tmm, archivekey, paramkey, tday, t1, t2, nuv, 'n', status )
    IF_NOTOK_RETURN(status=1)
    
    ! horizontal interpolation:
    call Transform2D( tmm, lli, nuv, tmi, status )
    IF_NOTOK_RETURN(status=1)
    
    ! expecting single level ...
    if ( size(tmm%llX,3) /= 1 ) then 
      write (gol,'("expecting single level:")'); call goErr
      write (gol,'("  paramkey  : ",a)') paramkey; call goErr
      write (gol,'("  levels    : ",a)') size(tmm%llX,3); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if
    
    ! extract first level
    ll = tmm%llX(:,:,1)
    
    !
    ! unit conversions, truncations, etc
    !
    
    select case ( paramkey )
      case ( 'oro', 'cp', 'lsp' )
        ! set minium; some negative values if made from spectral field:
        ll = max( 0.0, ll )
    end select

    !
    ! done
    !
    
    ! ok
    status = 0
    
  end subroutine tmm_ReadField_2d
  

  ! ***
  

  subroutine tmm_ReadField_3d( tmm, archivekey, paramkey, tday, t1, t2, &
                                    lli, nuv, levi, nw, sp, ll, tmi, status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol
    use Grid    , only : TllGridInfo, TLevelInfo
    use tmm_info, only : TMeteoInfo

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    character(len=*), intent(in)            ::  archivekey
    character(len=*), intent(in)            ::  paramkey
    type(TDate), intent(in)                 ::  tday, t1, t2
    type(TllGridInfo), intent(in)           ::  lli
    character(len=1), intent(in)            ::  nuv
    type(TLevelInfo), intent(in)            ::  levi 
    character(len=1), intent(in)            ::  nw    
    real, intent(out)                       ::  sp(:,:) 
    real, intent(out)                       ::  ll(:,:,:) 
    type(TMeteoInfo), intent(out)           ::  tmi
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_ReadField_3d'
    
    ! --- begin ----------------------------------
    
    call wrtgol( 'tmm read  : '//trim(paramkey)//' ', t1, '  -  ', t2 ); call goPr

    ! check shape of target grid:
    if ( ((nuv == 'n') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'u') .and. ((size(ll,1) /= lli%nlon+1) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'v') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat+1))) .or. &
         ((nuv == 'n') .and. ((size(sp,1) /= lli%nlon  ) .or. (size(sp,2) /= lli%nlat  ))) .or. &
         ((nuv == 'u') .and. ((size(sp,1) /= lli%nlon+1) .or. (size(sp,2) /= lli%nlat  ))) .or. &
         ((nuv == 'v') .and. ((size(sp,1) /= lli%nlon  ) .or. (size(sp,2) /= lli%nlat+1))) .or. &
         ((nw  == 'n') .and. (size(ll,3) /= levi%nlev  )) .or. &
         ((nw  == 'w') .and. (size(ll,3) /= levi%nlev+1)) ) then
      write (gol,'("target arrays do not match with grid definition:")')
      write (gol,'("  param  : ",a          )') paramkey   ; call goErr
      write (gol,'("  lli    : ",i3," x ",i3         )') lli%nlon, lli%nlat; call goErr
      write (gol,'("  nuv    : ",a                   )') nuv; call goErr
      write (gol,'("  levi   : ",i3                  )') levi%nlev; call goErr
      write (gol,'("  nw     : ",a                   )') nw; call goErr
      write (gol,'("  sp     : ",i3," x ",i3         )') shape(sp); call goErr
      write (gol,'("  ll     : ",i3," x ",i3," x ",i3)') shape(ll); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if

    !
    ! read, convert
    !
    
    ! read raw field in buffer
    call FillBuffer( tmm, archivekey, paramkey, tday, t1, t2, nuv, nw, status )
    IF_NOTOK_RETURN(status=1)

    ! 3d horizontal conversion:
    call Transform3Dh( tmm, lli, nuv, levi, nw, sp, tmi, status )
    IF_NOTOK_RETURN(status=1)
    
    ! 3d vertical conversion:
    call Transform3Dv( tmm, levi, nw, sp, ll, tmi, status )
    IF_NOTOK_RETURN(status=1)
    
    !
    ! unit conversion, extreme values
    !
    
    select case ( paramkey )
      case ( 'Q', 'CLWC', 'CIWC' )
        ! set minimum, stored values could be slightly negative
        ll = max( 0.0, ll )
        ll = max( 0.0, ll )
    end select

    !
    ! done
    !
    
    ! ok
    status = 0

  end subroutine tmm_ReadField_3d
  
  
  
  ! ==================================================================
  ! ===
  ! ===  VO/D/LNSP -> pu/pv/sp
  ! ===
  ! ==================================================================


  
  !
  ! 'tmpp:od-fc-ml60-glb3x2'
  ! 'prism:'
  !
 
  subroutine tmm_ReadUVSP( tmm, archivekey, tday, t1, t2, &
                                lli, levi, sp1, spm, sp2, spu, pu, spv, pv, &
                                sp1_tmi, spm_tmi, sp2_tmi, pu_tmi, pv_tmi, &
                                status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, operator(-), operator(+), operator(/), goSplitLine
    use Grid    , only : TllGridInfo, TLevelInfo
    use tmm_info, only : TMeteoInfo

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    type(TLevelInfo), intent(in)      ::  levi      
    real, intent(out)                 ::  sp1(:,:)       ! Pa
    real, intent(out)                 ::  spm(:,:)       ! Pa
    real, intent(out)                 ::  sp2(:,:)       ! Pa
    real, intent(out)                 ::  spu(:,:)       ! kg/s
    real, intent(out)                 ::  pu(:,:,:)      ! kg/s
    real, intent(out)                 ::  spv(:,:)       ! kg/s
    real, intent(out)                 ::  pv(:,:,:)      ! kg/s
    type(TMeteoInfo), intent(out)     ::  sp1_tmi, spm_tmi, sp2_tmi
    type(TMeteoInfo), intent(out)     ::  pu_tmi, pv_tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_ReadUVSP'
    
    ! --- local -------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=256)       ::  sourcename
    type(TDate)              ::  tmid

    ! --- begin -------------------------------
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
      
      case ( 'tmpp', 'tm5' )
      
        ! surface pressure at begin 6hr interval
        call ReadField( tmm, archivekey, 'sp', tday, t1, t1, &
                               lli, 'n', sp1, sp1_tmi, status )
        IF_NOTOK_RETURN(status=1)

        ! u flux valid for [t1,t2]
        call ReadField( tmm, archivekey, 'pu', tday, t1, t2, &
                               lli, 'u', levi, 'n', spu, pu, pu_tmi, status )
        IF_NOTOK_RETURN(status=1)
    
        ! v flux valid for [t1,t2]
        call ReadField( tmm, archivekey, 'pv', tday, t1, t2, &
                               lli, 'v', levi, 'n', spv, pv, pv_tmi, status )
        IF_NOTOK_RETURN(status=1)
    
        ! surface pressure at end 6hr interval
        call ReadField( tmm, archivekey, 'sp', tday, t2, t2, &
                               lli, 'n', sp2, sp2_tmi, status )
        IF_NOTOK_RETURN(status=1)
        
        ! mid of interval:
        tmid = t1 + (t2-t1)/2

        ! pressure at tmid, more or less valid for the whole interval ...
        call ReadField( tmm, archivekey, 'spm', tday, tmid, tmid, &
                               lli, 'n', spm, spm_tmi, status )
        IF_NOTOK_RETURN(status=1)
        
!#ifdef with_prism
!      case ( 'prism' )
!      
!        call tmm_ReadUVSP_raw( tmm, archivekey, tday, t1, t2, &
!                                 lli, levi, sp1, spm, sp2, pu, pv, &
!                                 sp1_tmi, spm_tmi, sp2_tmi, pu_tmi, pv_tmi, &
!                                 status )
!        IF_NOTOK_RETURN(status=1)
!#endif
        
      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_ReadUVSP


  ! ***
  

!#ifdef with_prism
!  subroutine tmm_ReadUVSP_raw( tmm, archivekey, tday, t1, t2, &
!                                 lli, levi, sp1, spm, sp2, pu, pv, &
!                                 sp1_tmi, spm_tmi, sp2_tmi, pu_tmi, pv_tmi
!                                 status )
!  
!    use Binas, only : R => ae
!    use Binas, only : g => grav
!    
!    use GO      , only : TDate, operator(+), operator(/), operator(-), rTotal
!    use Grid    , only : TllGridInfo, TLevelInfo
!    use Grid    , only : TshGridInfo, TshGrid, Init, Done, Set
!    use Grid    , only : IntArea, IntLat, IntLon, vod2uv, Nabla
!    use Grid    , only : BalanceMassFluxes, FillLevels, AreaOper
!    use tmm_info, only : TMeteoInfo, Init, Done, AddHistory
!
!    ! --- in/out --------------------------------
!  
!    type(TTmMeteo), intent(inout)     ::  tmm
!    character(len=*), intent(in)      ::  archivekey
!    type(TDate), intent(in)           ::  tday, t1, t2
!    type(TllGridInfo), intent(in)     ::  lli
!    type(TLevelInfo), intent(in)      ::  levi      
!    real, intent(out)                 ::  sp1(:,:)       ! Pa
!    real, intent(out)                 ::  spm(:,:)       ! Pa
!    real, intent(out)                 ::  sp2(:,:)       ! Pa
!    real, intent(out)                 ::  pu(:,:,:)      ! kg/s
!    real, intent(out)                 ::  pv(:,:,:)      ! kg/s
!    type(TMeteoInfo), intent(out)     ::  sp1_tmi, spm_tmi, sp2_tmi
!    type(TMeteoInfo), intent(out)     ::  pu_tmi, pv_tmi
!    integer, intent(out)              ::  status
!    
!    ! --- const --------------------------------------
!    
!    character(len=*), parameter  ::  rname = mname//'/tmm_Read_UVSP_raw'
!    
!    ! --- local -------------------------------
!    
!    type(TDate)               ::  tmid
!
!    integer                   ::  shT
!    integer                   ::  nlev
!    
!    ! spectral fields
!    type(TshGrid)             ::  LNSP1_sh, LNSPm_sh, LNSP2_sh
!    type(TshGrid), pointer    ::  D_sh(:), VO_sh(:)
!    type(TshGrid), pointer    ::  U_sh(:), V_sh(:)
!
!    ! integrated surface pressures:
!    real, allocatable         ::  sp(:,:,:)
!    
!    ! nabla.lnps
!    type(TshGrid)             ::  NabLNSP_sh(2)
!    
!    ! integrated Omega arrays:
!    real, allocatable         ::  IIOmega (:,:,:)
!    real, allocatable         ::  IIOmega2(:,:,:)
!    
!    ! pu/pv on source levels
!    real, allocatable         ::  puX(:,:,:)
!    real, allocatable         ::  pvX(:,:,:)
!    real, allocatable         ::  pwX(:,:,:)
!    
!    ! balance
!    real, allocatable         ::  dm_dt(:,:)
!    type(TllGridInfo)         ::  cgi
!    
!    ! loops etc
!    integer                   ::  l
!    
!    ! extra info
!    type(TMeteoInfo)          ::  lnsp1_tmi, lnspm_tmi, lnsp2_tmi
!    type(TMeteoInfo)          ::  vo_tmi, div_tmi, u_tmi, v_tmi
!    type(TMeteoInfo)          ::  pw_tmi
!    
!    ! --- begin -------------------------------
!    
!    ! mid of interval:
!    tmid = t1 + (t2-t1)/2
!
!    ! ----------------------------------------------------
!    ! read spectral fields
!    ! ----------------------------------------------------
!
!    !write (gol,'(a,": readspectral fiels")') name
!    
!    ! read LNSP t1
!    call FillBuffer( tmm, archivekey, 'LNSP', tday, t1, t1, 'n', status )
!    IF_NOTOK_RETURN(status=1)
!
!      call Init( LNSP1_sh )
!      call Set( LNSP1_sh, tmm%buf_shi%T, tmm%buf_sh(:,1) )
!      
!      lnsp1_tmi = tmm%buf_tmi
!      
!    ! read LNSP tmid
!    call FillBuffer( tmm, archivekey, 'LNSP', tday, tmid, tmid, 'n', status )
!    IF_NOTOK_RETURN(status=1)
!
!      call Init( LNSPm_sh )
!      call Set( LNSPm_sh, tmm%buf_shi%T, tmm%buf_sh(:,1) )
!      
!      lnspm_tmi = tmm%buf_tmi
!      
!    ! read VO tmid
!    call FillBuffer( tmm, archivekey, 'VO', tday, tmid, tmid, 'n', status )
!    IF_NOTOK_RETURN(status=1)
!
!      nlev = tmm%buf_levi%nlev
!      allocate( VO_sh(nlev) )
!      call Init( VO_sh )
!      do l = 1, nlev
!        call Set( VO_sh(l), tmm%buf_shi%T, tmm%buf_sh(:,l) )
!      end do
!      
!      vo_tmi = tmm%buf_tmi
!      
!    ! read D tmid
!    call FillBuffer( tmm, archivekey, 'D', tday, tmid, tmid, 'n', status )
!    IF_NOTOK_RETURN(status=1)
!
!      nlev = tmm%buf_levi%nlev
!      allocate( D_sh(nlev) )
!      call Init( D_sh )
!      do l = 1, nlev
!        call Set( D_sh(l), tmm%buf_shi%T, tmm%buf_sh(:,l) )
!      end do
!      
!      div_tmi = tmm%buf_tmi
!      
!    ! read LNSP t2
!    call FillBuffer( tmm, archivekey, 'LNSP', tday, t2, t2, 'n', status )
!    IF_NOTOK_RETURN(status=1)
!
!      call Init( LNSP2_sh )
!      call Set( LNSP2_sh, tmm%buf_shi%T, tmm%buf_sh(:,1) )
!     
!      lnsp2_tmi = tmm%buf_tmi
!      
!    ! check ...
!    nlev = tmm%buf_levi%nlev
!    if ( (size(VO_sh) /= nlev) .or. (size(D_sh) /= nlev) ) then
!      write (gol,'("number of levels should be the same:")')
!      write (gol,'("  vorticity  : ",i6)') size(VO_sh)
!      write (gol,'("  divergence : ",i6)') size(D_sh)
!      write (gol,'("  levels     : ",i6)') nlev
!      write (gol,'("in ",a)') rname; call goErr; status=1; return
!    end if
!    
!    ! check level order ..
!    !  ecmwf :  b =  0.0, .., 1.0
!    if ( tmm%buf_levi%b(nlev) < tmm%buf_levi%b(0) ) then
!      write (gol,'("please check implementation for level order bot -> top")')
!      write (gol,'("in ",a)') rname; call goErr; status=1; return
!    end if
!    
!    ! extract truncation
!    shT = tmm%buf_shi%T
!
!
!    !
!    ! allocate 3d fields
!    !
!    
!    allocate( IIOmega (lli%nlon,lli%nlat,nlev) )
!    allocate( IIOmega2(lli%nlon,lli%nlat,nlev) )
!    
!    allocate( puX(0:lli%nlon,  lli%nlat,nlev) )
!    allocate( pvX(  lli%nlon,0:lli%nlat,nlev) )
!    allocate( pwX(  lli%nlon,  lli%nlat,nlev) )
!    
!
!    !
!    ! compute U/V from VO/D
!    !
!
!    allocate( U_sh(nlev) )
!    allocate( V_sh(nlev) )
!
!    call Init( U_sh, shT )
!    call Init( V_sh, shT )
!
!    do l = 1, nlev
!      call vod2uv( VO_sh(l), D_sh(l), U_sh(l), V_sh(l) )
!    end do
!
!    ! history ...    
!    call Init( u_tmi, 'U', 'm/s', status, (/vo_tmi,div_tmi/) )
!    call Init( v_tmi, 'V', 'm/s', status, (/vo_tmi,div_tmi/) )
!    
!
!    ! ----------------------------------------------------
!    ! average surface pressures
!    ! ----------------------------------------------------
!    
!    !write (gol,'(a,": average surface pressure")') name
!    
!    allocate( sp(lli%nlon,lli%nlat,3) )
!
!    call IntArea( 'exp,aver', (/ LNSP1_sh, LNSPm_sh, LNSP2_sh /), lli, sp )
!    sp1 = sp(:,:,1)
!    spm = sp(:,:,2)
!    sp2 = sp(:,:,3)
!
!    deallocate( sp )
!    
!    call Init( sp1_tmi, 'sp', 'Pa', status, (/lnsp1_tmi/) )
!    call AddHistory( sp1_tmi, 'oper==exp,aver', status )
!    call Init( spm_tmi, 'sp', 'Pa', status, (/lnspm_tmi/) )
!    call AddHistory( spm_tmi, 'oper==exp,aver', status )
!    call Init( sp2_tmi, 'sp', 'Pa', status, (/lnsp2_tmi/) )
!    call AddHistory( sp2_tmi, 'oper==exp,aver', status )
!
!
!    ! ----------------------------------------------------
!    ! integrals over combination of spectral fields
!    ! ----------------------------------------------------
!    
!    !write (gol,'(a,": vertical flux")') name
!    
!    ! 
!    !   int int   D  (da+db*exp(LNSP)) cos(lat)  dA
!    !
!    !print *, '  pw'
!    
!    !print *, '    D'
!    IIOmega = 0.0
!    call IntArea( 'F*(da+db*exp(H))*cos', D_sh, LNSPm_sh, LNSPm_sh, &
!                  tmm%buf_levi%da, tmm%buf_levi%db, lli, IIOmega )
!    
!    !
!    !   int int   U  dLNSP1  exp(LNSP) db / cos(lat)  dA
!    !   int int   V  dLNSP2  exp(LNSP) db / cos(lat)  dA
!    !
!    
!    ! allocate
!    call Init( NabLNSP_sh(1) )
!    call Init( NabLNSP_sh(2) )
!    
!    ! compute nabla.lnsp for central time:
!    call Nabla( LNSPm_sh, NabLNSP_sh )
!
!    ! integral over cell area; add contributions
!    IIOmega2 = 0.0
!    !print *, '    U'
!    call IntArea( 'F*G*(db*exp(H))/cos', U_sh, NabLNSP_sh(1), LNSPm_sh, &
!                  tmm%buf_levi%da, tmm%buf_levi%db, lli, IIOmega2 )
!    !print *, '    V'
!    call IntArea( 'F*G*(db*exp(H))/cos', V_sh, NabLNSP_sh(2), LNSPm_sh, &
!                  tmm%buf_levi%da, tmm%buf_levi%db, lli, IIOmega2 )
!    
!    ! deallocate
!    call Done( NabLNSP_sh(1) )
!    call Done( NabLNSP_sh(2) )
!    
!    ! column integrated Omega
!
!    !print *, '    sum'
!    do l = 1, nlev    
!      ! replace with contribution of current level:
!      IIOmega(:,:,l) = (R**2)*IIOmega(:,:,l)/g  +  R*IIOmega2(:,:,l)/g
!      ! add contribution of previous levels:
!      if ( l > 1 ) then
!        IIOmega(:,:,l) = IIOmega(:,:,l) + IIOmega(:,:,l-1)
!      end if
!    end do
!    
!    ! compute vertical flux:
!    
!    do l = 1, nlev
!      pwX(:,:,l) = IIOmega(:,:,nlev) * tmm%buf_levi%b(l) - IIOmega(:,:,l)
!    end do
!    
!    call Init( pw_tmi, 'pw', 'kg/s', status, (/lnspm_tmi,div_tmi,u_tmi,v_tmi/) )
!    
!    ! ----------------------------------------------------
!    ! integrals over boundaries
!    ! ----------------------------------------------------
!    
!    !write (gol,'(a,": horizontal fluxes")') name
!    
!    !
!    !  pu  =  R/g int  U (da+db*exp(LNSP)) / cos(lat)  dlat
!    !
!    
!    !print *, '  pu'
!    call IntLat( '(da+exp*db)/cos', U_sh, LNSPm_sh, &
!                 tmm%buf_levi%da, tmm%buf_levi%db, lli, puX )
!    puX = puX * R/g
!
!    call Init( pu_tmi, 'pu', 'kg/s', (/u_tmi,lnspm_tmi/), status )
!    call AddHistory( pu_tmi, 'oper==intlat', status )
!
!    !
!    !  pv  =  R/g int  V (da+db*exp(LNSP)) dlon
!    !
!
!    !print *, '  pv'
!    call IntLon( '(da+exp*db)', V_sh, LNSPm_sh, &
!                 tmm%buf_levi%da, tmm%buf_levi%db, lli, pvX )
!    pvX = pvX * R/g
!
!    call Init( pv_tmi, 'pv', 'kg/s', (/v_tmi,lnspm_tmi/), status )
!    call AddHistory( pv_tmi, 'oper==intlon', status )
!
!
!    ! ----------------------------------------------------
!    ! balance
!    ! ----------------------------------------------------
!    
!    !write (gol,'(a,": balance")') name
!    
!    ! dummy coarse grid: global, single cell
!    call Init( cgi, 0.0, 360.0, 1, 0.0, 180.0, 1, status )
!    IF_NOTOK_RETURN(status=1)
!    
!    allocate( dm_dt(lli%nlon,lli%nlat) )
!
!    ! mass change at surface:
!    dm_dt = ( sp2 - sp1 ) / rTotal(t2-t1,'sec')     ! Pa/s
!    dm_dt = dm_dt / g                               ! kg/s/m2
!    call AreaOper( lli, dm_dt, '*', 'm2' )          ! kg/s
!    
!    ! balance pu/pv given pw and dmdt :
!    call BalanceMassFluxes( lli, tmm%buf_levi%nlev, &
!                                 puX, pvX, pwX, dm_dt, &
!                                 cgi, tmm%buf_levi%b )
!    
!    ! ok
!    call Done( cgi, status )
!    IF_NOTOK_RETURN(status=1)
!    
!    deallocate( dm_dt )
!    
!    call AddHistory( pu_tmi, 'oper==massbalance;', status )
!    call AddHistory( pu_tmi, pw_tmi, status )
!    call AddHistory( pu_tmi, sp1_tmi, status )
!    call AddHistory( pu_tmi, sp2_tmi, status )
!    call AddHistory( pv_tmi, 'oper==massbalance;', status )
!    call AddHistory( pv_tmi, pw_tmi, status )
!    call AddHistory( pv_tmi, sp1_tmi, status )
!    call AddHistory( pv_tmi, sp2_tmi, status )
!
!
!    ! ----------------------------------------------------
!    ! collect levels
!    ! ----------------------------------------------------
!
!    !write (gol,'(a,": collect levels")') name
!    
!    call FillLevels( levi, spm, pu, tmm%buf_levi, puX, 'sum', status )
!    IF_NOTOK_RETURN(status=1)
!    
!    call FillLevels( levi, spm, pv, tmm%buf_levi, pvX, 'sum', status )
!    IF_NOTOK_RETURN(status=1)
!    
!    call AddHistory( pu_tmi, 'oper==collectlevels', status )
!    call AddHistory( pv_tmi, 'oper==collectlevels', status )
!    
!
!    ! ----------------------------------------------------
!    ! done
!    ! ----------------------------------------------------
!
!    call Done( LNSP1_sh )
!    call Done( LNSPm_sh )
!    call Done( LNSP2_sh )
!
!    call Done( VO_sh )    
!    call Done(  D_sh )    
!    call Done(  U_sh )    
!    call Done(  V_sh )    
!    deallocate( VO_sh )
!    deallocate(  D_sh )
!    deallocate(  U_sh )
!    deallocate(  V_sh )
!
!    deallocate( IIOmega  )
!    deallocate( IIOmega2 )
!
!    deallocate( puX )
!    deallocate( pvX )
!    deallocate( pwX )
!    
!    call Done( lnsp1_tmi, status )
!    call Done( lnspm_tmi, status )
!    call Done( lnsp2_tmi, status )
!    call Done(    vo_tmi, status )
!    call Done(   div_tmi, status )
!    call Done(     u_tmi, status )
!    call Done(     v_tmi, status )
!    call Done(    pw_tmi, status )
!
!    !write (gol,'(a,": ok")') name
!    
!    ! ok
!    status = 0
!    
!  end subroutine tmm_ReadUVSP_raw
!#endif




  ! *****************************************************************
  

  subroutine tmm_Read_SP( tmm, archivekey, tday, t1, t2, &
                                lli, sp, tmi, status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, Get, wrtgol
    use GO      , only : goSplitLine
    use Grid    , only : TllGridInfo
    use Grid    , only : TshGridInfo, TshGrid, Init, Done, Set
    use Grid    , only : IntArea
    use tmm_info, only : TMeteoInfo, Init, AddHistory

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    real, intent(out)                 ::  sp(:,:)       ! Pa
    type(TMeteoInfo), intent(out)     ::  tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_SP'
    
    ! --- local -------------------------------
    
    character(len=10)         ::  sourcetype
    character(len=256)        ::  sourcename
    integer                   ::  hour

    ! --- begin -------------------------------
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from tmpp hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tmpp' )
      
        ! select parameter given hour:
        call Get( t1, hour=hour )
        if ( modulo(hour,6) == 3 ) then

          ! hour = 21/03/..  : uvsp files
          call ReadField( tmm, archivekey, 'sp', tday, t1, t1, &
                               lli, 'n', sp, tmi, status )
          IF_NOTOK_RETURN(status=1)

        else

          ! hour = 00/06/..  : spm files
          call ReadField( tmm, archivekey, 'spm', tday, t1, t1, &
                               lli, 'n', sp, tmi, status )
          IF_NOTOK_RETURN(status=1)

        end if
            
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from tm5 hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tm5' )
      
        ! sp always storred in 'sp' files ...
        call ReadField( tmm, archivekey, 'sp', tday, t1, t1, &
                             lli, 'n', sp, tmi, status )
        IF_NOTOK_RETURN(status=1)

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! convert spectral lnsp
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'ecmwf', 'ncep-cdc', 'ncep-gfs', 'msc-data', 'prism' )
      
        call wrtgol( 'tmm read  : sp ', t1, '  -  ', t2 ); call goPr

        ! read LNSP
        call FillBuffer( tmm, archivekey, 'LNSP', tday, t1, t1, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)
        
        ! should be spectral ...
        if ( tmm%buf_gridtype /= 'sh' ) then
          write (gol,'("expecting sh field, not ",a)') tmm%buf_gridtype
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! aera average exp(lnsp)
        call IntArea( 'exp,aver', tmm%buf_shi, tmm%buf_sh(:,1), lli, sp, status )
        IF_NOTOK_RETURN(status=1)
        
        ! set history info
        call Init( tmi, 'sp', 'Pa', status, (/tmm%buf_tmi/) )
        call AddHistory( tmi, 'oper==exp,aver', status )

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error ...
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_Read_SP


  ! *****************************************************************
  

  subroutine tmm_Read_MFUV( tmm, archivekey, tday, t1, t2, &
                                lli, levi, &
                                spu, mfu, mfu_tmi, &
                                spv, mfv, mfv_tmi, &
                                status )
  
    use Binas   , only : R => ae
    use Binas   , only : g => grav
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol
    use GO      , only : goSplitLine
    use Grid    , only : TllGridInfo, TLevelInfo
    use Grid    , only : TshGridInfo, TshGrid, Init, Done, Set
    use Grid    , only : IntLat, IntLon, vod2uv
    use Grid    , only : FillLevels
    use tmm_info, only : TMeteoInfo, Init, Done, AddHistory

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    type(TLevelInfo), intent(in)      ::  levi      
    real, intent(out)                 ::  spu(:,:) , spv(:,:)    ! Pa
    real, intent(out)                 ::  mfu(:,:,:), mfv(:,:,:)   ! kg/s
    type(TMeteoInfo), intent(out)     ::  mfu_tmi, mfv_tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_MFUV'
    
    ! --- local -------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=256)       ::  sourcename

    integer                   ::  shT
    integer                   ::  nlev
    
    ! spectral fields
    type(TshGrid)             ::  LNSP_sh
    type(TshGrid), pointer    ::  D_sh(:), VO_sh(:)
    type(TshGrid), pointer    ::  U_sh(:), V_sh(:)

    ! mfu/mfv on source levels
    real, allocatable         ::  mfuX(:,:,:)
    real, allocatable         ::  mfvX(:,:,:)
    
    ! loops etc
    integer                   ::  l
    
    ! extra info
    type(TMeteoInfo)          ::  LNSP_tmi
    type(TMeteoInfo)          ::  vo_tmi, div_tmi, u_tmi, v_tmi
    
    ! temporary arrays
    real, allocatable         ::  tmp_sp(:,:,:)
    
    ! --- begin -------------------------------
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
      
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tmpp', 'tm5' )

        ! flux through and surface pressure on west and east boundaries:      
        call ReadField( tmm, archivekey, 'mfu', tday, t1, t2, &
                               lli, 'u', levi, 'n', spu, mfu, mfu_tmi, status )
        IF_NOTOK_RETURN(status=1)
    
        ! flux through and surface pressure on south and north boundaries:      
        call ReadField( tmm, archivekey, 'mfv', tday, t1, t2, &
                               lli, 'v', levi, 'n', spv, mfv, mfv_tmi, status )
        IF_NOTOK_RETURN(status=1)
    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! convert spectral fields
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'ecmwf', 'ncep-cdc', 'ncep-gfs', 'msc-data' )
      
        call wrtgol( 'tmm read  : mfuv ', t1, '  -  ', t2 ); call goPr

        !
        ! read spectral fields
        !

        ! read VO
        call FillBuffer( tmm, archivekey, 'VO', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='sh' )
          IF_NOTOK_RETURN(status=1)

          ! extract grid size
          nlev = tmm%buf_levi%nlev
          shT  = tmm%buf_shi%T

          ! copy 3d spectral field from buffer:
          allocate( VO_sh(nlev) )
          call Init( VO_sh )
          do l = 1, nlev
            call Set( VO_sh(l), tmm%buf_shi%T, tmm%buf_sh(:,l) )
          end do
          vo_tmi = tmm%buf_tmi

          ! copy 1d spectral lnsp from buffer:
          call Init( LNSP_sh )
          call Set( LNSP_sh, tmm%buf_shi%T, tmm%buf_lnsp_sh )
          LNSP_tmi = tmm%buf_tmi

        ! read D
        call FillBuffer( tmm, archivekey, 'D', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='sh', shT=shT, nlev=nlev )
          IF_NOTOK_RETURN(status=1)

          ! copy 3d spectral field from buffer:
          nlev = tmm%buf_levi%nlev
          allocate( D_sh(nlev) )
          call Init( D_sh )
          do l = 1, nlev
            call Set( D_sh(l), tmm%buf_shi%T, tmm%buf_sh(:,l) )
          end do
          div_tmi = tmm%buf_tmi

        ! check ...
        nlev = tmm%buf_levi%nlev
        if ( (size(VO_sh) /= nlev) .or. (size(D_sh) /= nlev) ) then
          write (gol,'("number of levels should be the same:")')
          write (gol,'("  vorticity  : ",i6)') size(VO_sh)
          write (gol,'("  divergence : ",i6)') size(D_sh)
          write (gol,'("  levels     : ",i6)') nlev
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        !
        ! compute U/V from VO/D
        !

        allocate( U_sh(nlev) )
        allocate( V_sh(nlev) )

        call Init( U_sh, shT )
        call Init( V_sh, shT )

        do l = 1, nlev
          call vod2uv( VO_sh(l), D_sh(l), U_sh(l), V_sh(l) )
        end do
        
        ! history ...    
        call Init( u_tmi, 'U', 'm/s', status, (/vo_tmi,div_tmi/) )
        call Init( v_tmi, 'V', 'm/s', status, (/vo_tmi,div_tmi/) )

        !
        !  mfu  =  R/g int  U (da+db*exp(LNSP)) / cos(lat)  dlat
        !

        allocate( mfuX(0:lli%nlon,lli%nlat,nlev) )
        allocate( tmp_sp(0:lli%nlon,lli%nlat,1) )

        ! integral over boundary:
        call IntLat( '(da+exp*db)/cos', U_sh, LNSP_sh, &
                     tmm%buf_levi%da, tmm%buf_levi%db, lli, mfuX, status )
        IF_NOTOK_RETURN(status=1)
        mfuX = mfuX * R/g

        ! average surface pressure on boundary:
        call IntLat( 'exp(H),aver', (/LNSP_sh/), LNSP_sh, (/0.0/), (/0.0/), lli, tmp_sp, status )
        IF_NOTOK_RETURN(status=1)
        spu = tmp_sp(:,:,1)

        ! collect levels:
        call FillLevels( levi, 'n', spu, mfu, tmm%buf_levi, mfuX, 'sum', status )
        IF_NOTOK_RETURN(status=1)

        ! clear:
        deallocate( mfuX )
        deallocate( tmp_sp )

        ! info ...
        call Init( mfu_tmi, 'mfu', 'kg/s', status, (/u_tmi,LNSP_tmi/) )
        call AddHistory( mfu_tmi, 'oper==intlat', status )
        call AddHistory( mfu_tmi, 'oper==collectlevels', status )

        !
        !  mfv  =  R/g int  V (da+db*exp(LNSP)) dlon
        !

        allocate( mfvX(lli%nlon,0:lli%nlat,nlev) )
        allocate( tmp_sp(lli%nlon,0:lli%nlat,1) )

        ! integral over boundary:
        call IntLon( '(da+exp*db)', V_sh, LNSP_sh, &
                     tmm%buf_levi%da, tmm%buf_levi%db, lli, mfvX, status )
        IF_NOTOK_RETURN(status=1)
        mfvX = mfvX * R/g

        ! average surface pressure on boundary:
        call IntLon( 'exp(H),aver', (/LNSP_sh/), LNSP_sh, (/0.0/), (/0.0/), lli, tmp_sp, status )
        IF_NOTOK_RETURN(status=1)
        spv = tmp_sp(:,:,1)

        ! collect levels:
        call FillLevels( levi, 'n', spv, mfv, tmm%buf_levi, mfvX, 'sum', status )
        IF_NOTOK_RETURN(status=1)

        ! clear:
        deallocate( mfvX )
        deallocate( tmp_sp )

        ! info ...
        call Init( mfv_tmi, 'mfv', 'kg/s', status, (/v_tmi,LNSP_tmi/) )
        call AddHistory( mfv_tmi, 'oper==intlon', status )
        call AddHistory( mfv_tmi, 'oper==collectlevels', status )
        
        !
        ! done
        !

        call Done( LNSP_sh )

        call Done( VO_sh )    
        call Done(  D_sh )    
        call Done(  U_sh )    
        call Done(  V_sh )    

        deallocate( VO_sh )
        deallocate(  D_sh )
        deallocate(  U_sh )
        deallocate(  V_sh )

        call Done( lnsp_tmi, status )
        call Done(   vo_tmi, status )
        call Done(  div_tmi, status )
        call Done(    u_tmi, status )
        call Done(    v_tmi, status )

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_Read_MFUV


  ! *****************************************************************
  

  subroutine tmm_Read_MFW( tmm, archivekey, tday, t1, t2, lli, levi, &
                                sp, mfw, tmi, status )
  
    use Binas   , only : R => ae
    use Binas   , only : g => grav
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol
    use GO      , only : goSplitLine
    use Grid    , only : TllGridInfo, TLevelInfo
    use Grid    , only : TshGridInfo, TshGrid, Init, Done, Set
    use Grid    , only : vod2uv, Nabla, IntArea
    use Grid    , only : FillLevels
    use tmm_info, only : TMeteoInfo, Init, Done, AddHistory

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    type(TLevelInfo), intent(in)      ::  levi      
    real, intent(out)                 ::  sp(:,:)      ! Pa
    real, intent(out)                 ::  mfw(:,:,:)   ! kg/s
    type(TMeteoInfo), intent(out)     ::  tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_MFW'
    
    ! --- local -------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=256)       ::  sourcename

    integer                   ::  shT
    integer                   ::  nlev
    
    ! spectral fields
    type(TshGrid)             ::  LNSP_sh
    type(TshGrid), pointer    ::  D_sh(:), VO_sh(:)
    type(TshGrid), pointer    ::  U_sh(:), V_sh(:)

    ! nabla.lnps
    type(TshGrid)             ::  NabLNSP_sh(2)
    
    ! integrated Omega arrays:
    real, allocatable         ::  IIOmega (:,:,:)
    real, allocatable         ::  IIOmega2(:,:,:)
    
    ! mfw on source levels
    real, allocatable         ::  mfwX(:,:,:)
    
    ! loops etc
    integer                   ::  l
    
    ! extra info
    type(TMeteoInfo)          ::  LNSP_tmi
    type(TMeteoInfo)          ::  vo_tmi, div_tmi, u_tmi, v_tmi
    
    ! --- begin -------------------------------
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
      
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tmpp', 'tm5' )
      
        ! downward mass flux through bottom:
        call ReadField( tmm, archivekey, 'mfw', tday, t1, t2, &
                               lli, 'n', levi, 'w', sp, mfw, tmi, status )
        IF_NOTOK_RETURN(status=1)

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! convert spectral fields
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'ecmwf', 'ncep-cdc', 'ncep-gfs', 'msc-data' )
      
        call wrtgol( 'tmm read  : mfw ', t1, '  -  ', t2 ); call goPr

        !
        ! read spectral fields
        !

        ! read VO
        call FillBuffer( tmm, archivekey, 'VO', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='sh' )
          IF_NOTOK_RETURN(status=1)

          ! extract grid size
          nlev = tmm%buf_levi%nlev
          shT  = tmm%buf_shi%T

          ! copy 3d spectral field from buffer:
          allocate( VO_sh(nlev) )
          call Init( VO_sh )
          do l = 1, nlev
            call Set( VO_sh(l), tmm%buf_shi%T, tmm%buf_sh(:,l) )
          end do
          vo_tmi = tmm%buf_tmi

          ! copy 1d spectral lnsp from buffer:
          call Init( LNSP_sh )
          call Set( LNSP_sh, tmm%buf_shi%T, tmm%buf_lnsp_sh )
          LNSP_tmi = tmm%buf_tmi

        ! read D
        call FillBuffer( tmm, archivekey, 'D', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='sh', shT=shT, nlev=nlev )
          IF_NOTOK_RETURN(status=1)

          ! copy 3d spectral field from buffer:
          nlev = tmm%buf_levi%nlev
          allocate( D_sh(nlev) )
          call Init( D_sh )
          do l = 1, nlev
            call Set( D_sh(l), tmm%buf_shi%T, tmm%buf_sh(:,l) )
          end do
          div_tmi = tmm%buf_tmi

        ! check ...
        nlev = tmm%buf_levi%nlev
        if ( (size(VO_sh) /= nlev) .or. (size(D_sh) /= nlev) ) then
          write (gol,'("number of levels should be the same:")'); call goErr
          write (gol,'("  vorticity  : ",i6)') size(VO_sh); call goErr
          write (gol,'("  divergence : ",i6)') size(D_sh); call goErr
          write (gol,'("  levels     : ",i6)') nlev; call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! extract truncation
        shT = tmm%buf_shi%T

        !
        ! compute U/V from VO/D
        !

        allocate( U_sh(nlev) )
        allocate( V_sh(nlev) )

        call Init( U_sh, shT )
        call Init( V_sh, shT )

        do l = 1, nlev
          call vod2uv( VO_sh(l), D_sh(l), U_sh(l), V_sh(l) )
        end do

        ! history ...    
        call Init( u_tmi, 'U', 'm/s', status, (/vo_tmi,div_tmi/) )
        call Init( v_tmi, 'V', 'm/s', status, (/vo_tmi,div_tmi/) )


        !
        ! allocate 3d fields
        !

        allocate( IIOmega (lli%nlon,lli%nlat,nlev) )
        allocate( IIOmega2(lli%nlon,lli%nlat,nlev) )

        allocate( mfwX(lli%nlon,lli%nlat,nlev+1) )


        ! 
        !   int int   D  (da+db*exp(LNSP)) cos(lat)  dA
        !

        IIOmega = 0.0
        call IntArea( 'F*(da+db*exp(H))*cos', D_sh, LNSP_sh, LNSP_sh, &
                      tmm%buf_levi%da, tmm%buf_levi%db, lli, IIOmega, status )
        IF_NOTOK_RETURN(status=1)

        !
        !   int int   U  dLNSP1  exp(LNSP) db / cos(lat)  dA
        !   int int   V  dLNSP2  exp(LNSP) db / cos(lat)  dA
        !

        ! allocate
        call Init( NabLNSP_sh(1) )
        call Init( NabLNSP_sh(2) )

        ! compute nabla.lnsp :
        call Nabla( LNSP_sh, NabLNSP_sh )

        ! integral over cell area; add contributions
        IIOmega2 = 0.0
        call IntArea( 'F*G*(db*exp(H))/cos', U_sh, NabLNSP_sh(1), LNSP_sh, &
                      tmm%buf_levi%da, tmm%buf_levi%db, lli, IIOmega2, status )
        IF_NOTOK_RETURN(status=1)
        !print *, '    V'
        call IntArea( 'F*G*(db*exp(H))/cos', V_sh, NabLNSP_sh(2), LNSP_sh, &
                      tmm%buf_levi%da, tmm%buf_levi%db, lli, IIOmega2, status )
        IF_NOTOK_RETURN(status=1)

        ! deallocate
        call Done( NabLNSP_sh(1) )
        call Done( NabLNSP_sh(2) )

        !
        ! column integrated Omega
        !

        ! parent levels downwards or upwards ?
        if ( tmm%buf_levi%updo == 'd' ) then

          ! loop from top to bottom:
          do l = 1, nlev    
            ! replace with contribution of current level:
            IIOmega(:,:,l) = (R**2)*IIOmega(:,:,l)/g  +  R*IIOmega2(:,:,l)/g
            ! add contribution of previous levels:
            if ( l > 1 ) then
              IIOmega(:,:,l) = IIOmega(:,:,l) + IIOmega(:,:,l-1)
            end if
          end do

        else

          ! loop from top to bottom:
          do l = nlev, 1, -1
            ! replace with contribution of current level:
            IIOmega(:,:,l) = (R**2)*IIOmega(:,:,l)/g  +  R*IIOmega2(:,:,l)/g
            ! add contribution of levels above:
            if ( l < nlev ) then
              IIOmega(:,:,l) = IIOmega(:,:,l) + IIOmega(:,:,l+1)
            end if
          end do

        end if

        !
        ! compute vertical flux:
        !

        ! parent levels downwards or upwards ?
        if ( tmm%buf_levi%updo == 'd' ) then

          ! top hlev:
          mfwX(:,:,1) = 0.0    ! kg/s
          ! loop from top to bottom layer:
          do l = 1, nlev
            ! lay l bot hlev    surflay bot hlev    lay l bot hlev          lay l bot hlev
            ! 2 .. nlev+1       nlev                0 .. nlev-1             1 .. nlev
            mfwX(:,:,l+1)   =   IIOmega(:,:,nlev) * tmm%buf_levi%b(l)   -   IIOmega(:,:,l)
          end do
          
        else

          ! top hlev:
          mfwX(:,:,nlev+1) = 0.0    ! kg/s
          ! loop from top to bottom layer:
          do l = nlev, 1, -1
            ! lay l bot hlev    surflay bot hlev   lay l bot hlev            lay l bot hlev
            ! 1 .. nlev         nlev               0 .. nlev-1               1 .. nlev
            mfwX(:,:,l)    =    IIOmega(:,:,1)  *  tmm%buf_levi%b(l-1)   -   IIOmega(:,:,l)
          end do
          
        end if
        
        ! check: fluxh through bottom should be zero ...
        if ( maxval(abs(mfwX(:,:,1))) > 1.0 ) then
          write (gol,'("vert.flux through bottom half level should be zero ...")'); call goErr
          write (gol,'("  max value  : ",es12.4)') maxval(abs(mfwX(:,:,1))); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if
        
        ! create history
        call Init( tmi, 'mfw', 'kg/s', status, (/lnsp_tmi,div_tmi,u_tmi,v_tmi/) )
        
        
        !
        ! collect levels
        !
        
        ! average surface pressure
        call IntArea( 'exp,aver', tmm%buf_shi, LNSP_sh%c, lli, sp, status )
        IF_NOTOK_RETURN(status=1)

        ! combine levels etc
        call FillLevels( levi, 'w', sp, mfw, tmm%buf_levi, mfwX, 'bottom', status )
        IF_NOTOK_RETURN(status=1)
    
        call AddHistory( tmi, 'oper==collectlevels', status )
        
        !
        ! upward flux
        !
        
        ! flux should be upwards (in direction of increasing level):
        mfw = - mfw
        
        call AddHistory( tmi, 'oper==upwards', status )
        

        !
        ! done
        !

        call Done( LNSP_sh )

        call Done( VO_sh )    
        call Done(  D_sh )    
        call Done(  U_sh )    
        call Done(  V_sh )    

        deallocate( VO_sh )
        deallocate(  D_sh )
        deallocate(  U_sh )
        deallocate(  V_sh )
        
        deallocate( IIOmega  )
        deallocate( IIOmega2 )
        deallocate( mfwX      )

        call Done( lnsp_tmi, status )
        call Done(   vo_tmi, status )
        call Done(  div_tmi, status )
        call Done(    u_tmi, status )
        call Done(    v_tmi, status )

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_Read_MFW


  ! *****************************************************************
  

  subroutine tmm_Read_TQ( tmm, archivekey_T, archivekey_Q, tday, t1, t2, lli, levi, &
                                sp, T, T_tmi, Q, Q_tmi, status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol
    use GO      , only : goSplitLine
    use Phys    , only : RealTemperature
    use Grid    , only : TllGridInfo, TLevelInfo
    use tmm_info, only : TMeteoInfo, Init, Done, AddHistory

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey_T
    character(len=*), intent(in)      ::  archivekey_Q
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    type(TLevelInfo), intent(in)      ::  levi      
    real, intent(out)                 ::  sp(:,:)      ! Pa
    real, intent(out)                 ::  T(:,:,:)     ! K
    type(TMeteoInfo), intent(out)     ::  T_tmi
    real, intent(out)                 ::  Q(:,:,:)     ! kg/kg
    type(TMeteoInfo), intent(out)     ::  Q_tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_TQ'
    
    ! --- local -------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=256)       ::  sourcename

    ! --- begin -------------------------------
    
    !
    ! read humidity
    !

    ! just read ...
    call ReadField( tmm, archivekey_Q, 'Q', tday, t1, t2, &
                           lli, 'n', levi, 'n', sp, Q, Q_tmi, status )
    IF_NOTOK_RETURN(status=1)

    !
    ! read temperature
    !

    ! split source key in type and name:
    call goSplitLine( archivekey_T, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
      
      case ( 'tmpp', 'tm5', 'ecmwf', 'msc-data' )
      
        ! temperature:
        call ReadField( tmm, archivekey_T, 'T', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, T, T_tmi, status )
        IF_NOTOK_RETURN(status=1)

      case ( 'ncep-cdc', 'ncep-gfs' )
      
        ! virtual temperature:
        call ReadField( tmm, archivekey_T, 'Tv', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, T, T_tmi, status )
        IF_NOTOK_RETURN(status=1)

        ! convert:
        T = RealTemperature( T, Q )
        
        ! info:
        call AddHistory( T_tmi, 'convert from virtual temperature', status )

      case default
      
        write (gol,'("unsupported temper source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    !
    ! done
    !

    ! ok
    status = 0
    
  end subroutine tmm_Read_TQ


  ! *****************************************************************
  

  subroutine tmm_Read_CloudCovers( tmm, archivekey, tday, t1, t2, lli, levi, &
                        sp, cc, cc_tmi, cco, cco_tmi, ccu, ccu_tmi, status )
  
    use GO       , only : goErr, gol, goPr
    use GO       , only : TDate, wrtgol, gol, goPr
    use GO       , only : goSplitLine
    use Grid     , only : TllGridInfo, TLevelInfo, FillLevels
    use Phys     , only : cf_overhead
    use tmm_info , only : TMeteoInfo, AddHistory
    use tmm_param, only : GetCombineKeys

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    type(TLevelInfo), intent(in)      ::  levi      
    real, intent(out)                 ::  sp(:,:)      ! Pa
    real, intent(out)                 ::  cc(:,:,:), cco(:,:,:), ccu(:,:,:)   ! 0-1
    type(TMeteoInfo), intent(out)     ::  cc_tmi, cco_tmi, ccu_tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_CloudCovers'
    
    ! --- local -------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=256)       ::  sourcename
    
    integer                  ::  i, j, l, lme
    real, allocatable        ::   cc_col(:)
    real, allocatable        ::  cco_col(:), ccoX(:,:,:)
    real, allocatable        ::  ccu_col(:), ccuX(:,:,:)
    character(len=10)        ::  hcomb, vcomb
    
    ! --- begin -------------------------------
    
    ! check ...
    if ( any(shape(cco)/=shape(cc)) .or. any(shape(ccu)/=shape(cc)) ) then
      write (gol,'("output arrays should have same shape:")'); call goErr
      write (gol,'("   cc : ",3i4)') shape( cc); call goErr
      write (gol,'("  cco : ",3i4)') shape(cco); call goErr
      write (gol,'("  ccu : ",3i4)') shape(ccu); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
      
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tmpp', 'tm5' )
      
        ! cloud cover
        call ReadField( tmm, archivekey, 'CC', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, cc,  cc_tmi, status )
        IF_NOTOK_RETURN(status=1)

        ! cloud cover overhead
        call ReadField( tmm, archivekey, 'CCO', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, cco, cco_tmi, status )
        IF_NOTOK_RETURN(status=1)
        
        ! cloud cover underfeet
        call ReadField( tmm, archivekey, 'CCU', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, ccu, ccu_tmi, status )
        IF_NOTOK_RETURN(status=1)
        
        ! set extrema; stored values could be slightly outside [0,1]
         cc = max( 0.0, min(  cc, 1.0 ) )
        cco = max( 0.0, min( cco, 1.0 ) )
        ccu = max( 0.0, min( ccu, 1.0 ) )

    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! convert from raw gg fields
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'ecmwf' )
      
        call wrtgol( 'tmm read  : cc   ', t1, '  -  ', t2 ); call goPr
        
        !
        ! read cc: gg, all levels
        !
        
        call FillBuffer( tmm, archivekey, 'CC', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)
        
        ! 3d horizontal conversion; result is stored in tmm%llX
        call Transform3Dh( tmm, lli, 'n', levi, 'n', sp, cc_tmi, status )
        IF_NOTOK_RETURN(status=1)

        !
        ! compute cloudcover overhead using all levels
        !
        
        ! unreduced number of levels:
        lme = tmm%buf_levi%nlev
        
        ! storage:
        allocate(  cc_col(lme) )
        allocate( cco_col(lme) )
        allocate( ccoX(lli%nlon,lli%nlat,lme) )
        allocate( ccu_col(lme) )
        allocate( ccuX(lli%nlon,lli%nlat,lme) )

        ! loop over grid cells
        do j = 1, lli%nlat
          do i = 1, lli%nlon

            ! overhead cloud cover:
            cc_col = tmm%llX(i,j,:)
            call cf_overhead ( lme, cc_col, cco_col )
            ccoX(i,j,:) = cco_col
            
            ! underfeet cloud cover; first reverse layers:
            do l = 1, lme
              cc_col(l) = tmm%llX(i,j,lme+1-l)
            end do
            call cf_overhead( lme, cc_col, ccu_col )
            do l = 1, lme
              ccuX(i,j,l) = ccu_col(lme+1-l)
            end do

          end do
        end do
        
        ! clear
        deallocate(  cc_col )
        deallocate( cco_col )
        deallocate( ccu_col )

        ! info on this operation:        
        call AddHistory( cco_tmi, 'oper==cf_overhead', status )

        !
        ! 3d vertical conversions
        !

        ! convert from tmm%buf_llX to cc
        call Transform3Dv( tmm, levi, 'n', sp, cc, cc_tmi, status )
        IF_NOTOK_RETURN(status=1)

        ! store ccoX in buffer, convert from tmm%llX to cco
        call GetCombineKeys( hcomb, vcomb, 'cco', status )
        IF_NOTOK_RETURN(status=1)
        call FillLevels( levi, 'n', sp, cco, tmm%buf_levi, ccoX, vcomb, status )
        IF_NOTOK_RETURN(status=1)
        call AddHistory( cco_tmi, 'oper==vcomb,'//trim(vcomb), status )
        
        ! store ccuX in buffer, convert from tmm%llX to ccu
        call GetCombineKeys( hcomb, vcomb, 'ccu', status )
        IF_NOTOK_RETURN(status=1)
        call FillLevels( levi, 'n', sp, ccu, tmm%buf_levi, ccuX, vcomb, status )
        IF_NOTOK_RETURN(status=1)
        call AddHistory( ccu_tmi, 'oper==vcomb,'//trim(vcomb), status )
        
        ! clear
        deallocate( ccoX )

        ! set extrema; stored values could be slightly outside [0,1]
         cc = max( 0.0, min(  cc, 1.0 ) )
        cco = max( 0.0, min( cco, 1.0 ) )
        ccu = max( 0.0, min( ccu, 1.0 ) )

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_Read_CloudCovers


  ! *****************************************************************
  

  subroutine tmm_Read_Convec( tmm, archivekey, tday, t1, t2, lli, levi, &
                                omega, omega_tmi, &
                                sp, &
                                entu, entu_tmi, entd, entd_tmi, &
                                detu, detu_tmi, detd, detd_tmi, &
                                status )

    use Binas   , only : grav  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol, Get, IncrDate
    use GO      , only : operator(-), operator(+), rTotal
    use GO      , only : goSplitLine
    use Phys    , only : mid2bound_uv, mid2bound_w, mid2bound_t
    use Phys    , only : mid2bound_q, mid2bound_z, mid2bound_p
    use Phys    , only : subscal, phlev_ec_rev, geopot
    use Phys    , only : RealTemperature
    use Grid    , only : TllGridInfo
    use Grid    , only : TLevelInfo, HPressure, FPressure, FillLevels
    use Grid    , only : TShGrid, VoD2UV
    use Grid    , only : TggGridInfo, InterpolMask, Divergence, Tgg2llFracs, FracSum, AreaOper
    use Grid    , only : Init, Done, Set, Interpol
    use tmm_info, only : TMeteoInfo, Init, AddHistory

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    type(TLevelInfo), intent(in)      ::  levi
    real, intent(in)                  ::  omega(:,:,:)   ! Pa/s
    type(TMeteoInfo), intent(in)      ::  omega_tmi
    real, intent(out)                 ::  sp(:,:)      ! Pa
    real, intent(out)                 ::  entu(:,:,:), entd(:,:,:)   ! kg/m2/s
    type(TMeteoInfo), intent(out)     ::  entu_tmi, entd_tmi
    real, intent(out)                 ::  detu(:,:,:), detd(:,:,:)   ! kg/m2/s
    type(TMeteoInfo), intent(out)     ::  detu_tmi, detd_tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_Convec'
    
    ! --- local -------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=256)       ::  sourcename

    integer                  ::  lout
    real, allocatable        ::  ll(:,:,:)
    
    ! timing
    integer                  ::  hour
    type(TDate)              ::  t1s, t2s
    
    ! loops etc
    integer                  ::  l, nlev
    integer                  ::  i, i1, i2, j
    
    ! spectral fields
    integer                  ::  shT
    type(TshGrid)            ::  LNSP_sh
    type(TshGrid), pointer   ::  D_sh(:), VO_sh(:)
    type(TshGrid)            ::  U_sh, V_sh

    ! gaussian grids
    integer                  ::  ggN
    logical                  ::  reduced
    type(TggGridInfo)        ::  ggi
    real, allocatable        ::  gg(:)
    real, allocatable        ::  slhf_gg(:)
    real, allocatable        ::   oro_gg(:)
    real, allocatable        ::    sp_gg(:)
    real, allocatable        ::     m_gg(:)
    real, allocatable        ::  u_gg(:,:)
    real, allocatable        ::  v_gg(:,:)
    real, allocatable        ::  w_gg(:,:)
    real, allocatable        ::  t_gg(:,:)
    real, allocatable        ::  q_gg(:,:)
    real, allocatable        ::  p_gg(:,:)
    real, allocatable        ::  z_gg(:,:)
    logical, allocatable     ::  ggmask(:)
    real, allocatable        ::  qam_gg(:,:), qac_gg(:,:)
    real, allocatable        ::  entu_gg(:,:)
    real, allocatable        ::  detu_gg(:,:)
    real, allocatable        ::  entd_gg(:,:)
    real, allocatable        ::  detd_gg(:,:)
    type(Tgg2llFracs)        ::  gg2ll
    real, allocatable        ::  llX(:,:,:)
    
    ! subgrid stuff
    real                     ::  dt_sec
    real                     ::  evap
    real, allocatable        ::  p_hlev(:)

    ! extra info
    type(TMeteoInfo)         ::  slhf_tmi, sp_tmi, oro_tmi
    type(TMeteoInfo)         ::  vo_tmi, div_tmi, u_tmi, v_tmi
    type(TMeteoInfo)         ::  w_tmi, t_tmi, q_tmi
    
    ! reversed levels
    type(TLevelInfo)         ::  leviX
    real, allocatable        ::  aX(:), bX(:)
    real, allocatable        ::  tmp_gg(:,:)
    
    ! --- begin -------------------------------
    
    ! number of levels in output arrays:
    lout = size(entu,3)
    
    ! check ...
    if ( (size(entd,3)/=lout) .or. &
         (size(detu,3)/=lout) .or. (size(detd,3)/=lout) ) then
      write (gol,'("output arrays should have same number of levels:")'); call goErr
      write (gol,'("  entu : ",i4)') size(entu,3); call goErr
      write (gol,'("  entd : ",i4)') size(entd,3); call goErr
      write (gol,'("  detu : ",i4)') size(detu,3); call goErr
      write (gol,'("  detd : ",i4)') size(detd,3); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
      
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tmpp', 'tm5' )
      
        ! full level output:
        allocate( ll(lli%nlon,lli%nlat,levi%nlev) )

        ! entrement updraught
        call ReadField( tmm, archivekey, 'eu', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, ll, entu_tmi, status )
        IF_NOTOK_RETURN(status=1)
        !
        entu = ll(:,:,1:lout)

        ! entrement downdraught
        call ReadField( tmm, archivekey, 'ed', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, ll, entd_tmi, status )
        IF_NOTOK_RETURN(status=1)
        !
        entd = ll(:,:,1:lout)
        
        ! detrement updraught
        call ReadField( tmm, archivekey, 'du', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, ll, detu_tmi, status )
        IF_NOTOK_RETURN(status=1)
        !
        detu = ll(:,:,1:lout)
    
        ! detrement downdraught
        call ReadField( tmm, archivekey, 'dd', tday, t1, t2, &
                               lli, 'n', levi, 'n', sp, ll, detd_tmi, status )
        IF_NOTOK_RETURN(status=1)
        !
        detd = ll(:,:,1:lout)
        
        ! clear
        deallocate( ll )
    
    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! convert from raw fields (sh,gg)
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'ecmwf', 'ncep-cdc', 'ncep-gfs', 'msc-data' )
      
        ! only hours 00/03/06/etc yet ...
        call Get( t1, hour=hour )
        if ( modulo(hour,3) /= 0 ) then
          write (gol,'("convection only for 6hr intervals around 00/06/12/18 yet ...")'); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if
        
        ! time step in sec:
        !dt_sec = rTotal( t2 - t1, 'sec' )
        
        ! use 6 hour interval around requested (instant!) time to read slhf;
        ! for 3 hourly request, this means double reading
        ! (no problem since slhf is time average anyway)
        dt_sec = 6 * 3600.0

        !
        ! slhf (gg)
        !
        
        ! first interval: [t1-dt/2,t1]
        t1s = t1 - IncrDate(sec=int(dt_sec/2))
        t2s = t1

        call wrtgol( 'tmm read  : slhf ', t1s, '  -  ', t2s ); call goPr

        ! read first slhf in buffer (W/m2 time aver):        
        call FillBuffer( tmm, archivekey, 'slhf', tday, t1s, t2s, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='gg' )
          IF_NOTOK_RETURN(status=1)

          ! extract grid size
          ggN     = tmm%buf_ggi%N
          reduced = tmm%buf_ggi%reduced

          ! setup gg defintion from buffer:
          call Init( ggi, ggN, reduced, status )
          IF_NOTOK_RETURN(status=1)

          ! allocate storage:
          allocate(  oro_gg(ggi%np) )
          allocate( slhf_gg(ggi%np) )
          allocate(   sp_gg(ggi%np) )

          ! store first half of slhf (accumulated over 3 hr):
          slhf_gg  = tmm%buf_gg(:,1) * dt_sec/2 ! W/m2 s
          slhf_tmi = tmm%buf_tmi

        ! second interval: [t1,t1+dt/2]
        t1s = t1
        t2s = t1 + IncrDate(sec=int(dt_sec/2))

        call wrtgol( 'tmm read  : slhf ', t1s, '  -  ', t2s ); call goPr

        ! read first slhf in buffer (W/m2 time aver):      
        call FillBuffer( tmm, archivekey, 'slhf', tday, t1s, t2s, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='gg', np=ggi%np )
          IF_NOTOK_RETURN(status=1)

          ! add second half of slhf (accumulated over 3 hr):
          slhf_gg = slhf_gg + tmm%buf_gg(:,1) * dt_sec/2   ! W/m2 s

          ! copy info
          call AddHistory( slhf_tmi, tmm%buf_tmi, status )

        
        !
        ! orography (gg)
        !
        
        write (gol,'("    tmm read  : oro")'); call goPr

        ! read orography in buffer:
        call FillBuffer( tmm, archivekey, 'oro', tday, t1, t1, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! interpol from sh ?
          select case ( tmm%buf_gridtype )
            case ( 'gg' )
              ! copy from buffer:
              oro_gg  = tmm%buf_gg(:,1)  ! m*[g]
            case ( 'sh' )
              ! interpol from sh to gg :
              call Interpol( tmm%buf_shi, tmm%buf_sh(:,1), ggi, oro_gg, status )
              IF_NOTOK_RETURN(status=1)
            case default
              write (gol,'("unsupported grid type `",a,"` for raw oro")') tmm%buf_gridtype
              write (gol,'("in ",a)') rname; call goErr; status=1; return
          end select

          ! store:
          oro_tmi = tmm%buf_tmi

        !
        ! read raw 3d fields
        !

        ! ~~~

        call wrtgol( 'tmm read  : u,v  ', t1, '  -  ', t2 ); call goPr

        ! read VO
        call FillBuffer( tmm, archivekey, 'VO', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='sh' )
          IF_NOTOK_RETURN(status=1)
          
          ! extract other grid sizes
          shT = tmm%buf_shi%T
          nlev = tmm%buf_levi%nlev

          ! allocate 3d storage:
          allocate( u_gg(ggi%np,0:nlev) ) ; u_gg = 0.0
          allocate( v_gg(ggi%np,0:nlev) ) ; v_gg = 0.0
          allocate( w_gg(ggi%np,0:nlev) ) ; w_gg = 0.0
          allocate( t_gg(ggi%np,0:nlev) ) ; t_gg = 0.0
          allocate( q_gg(ggi%np,0:nlev) ) ; q_gg = 0.0
          allocate( p_gg(ggi%np,0:nlev) ) ; p_gg = 0.0
          allocate( z_gg(ggi%np,0:nlev) ) ; z_gg = 0.0

          ! copy 3d spectral field from buffer:
          allocate( VO_sh(nlev) )
          call Init( VO_sh )
          do l = 1, nlev
            call Set( VO_sh(l), shT, tmm%buf_sh(:,l) )
          end do
          
          ! copy info
          vo_tmi = tmm%buf_tmi

        ! read D
        call FillBuffer( tmm, archivekey, 'D', tday, t1, t2, 'n', 'n', status )
        IF_NOTOK_RETURN(status=1)

          ! check ...
          call CheckBuffer( tmm, status, gridtype='sh', shT=shT, nlev=nlev )
          IF_NOTOK_RETURN(status=1)
          
          ! copy 3d spectral field from buffer:
          allocate( D_sh(nlev) )
          call Init( D_sh )
          do l = 1, nlev
            call Set( D_sh(l), shT, tmm%buf_sh(:,l) )
          end do
          
          ! copy info
          div_tmi = tmm%buf_tmi
          
        ! convert from sh VO/D to gg U/V
        ! setup storage:
        call Init( U_sh, shT )
        call Init( V_sh, shT )
        ! loop over levels:
        do l = 1, nlev
          ! convert to U=u*cos(lat) and V=v*cos(lat) :
          call vod2uv( VO_sh(l), D_sh(l), U_sh, V_sh )
          ! convert to Gaussian grid:
          call Interpol( U_sh, ggi, u_gg(:,l), status )
          IF_NOTOK_RETURN(status=1)
          call Interpol( V_sh, ggi, v_gg(:,l), status )
          IF_NOTOK_RETURN(status=1)
        end do
        ! clear
        call Done(  U_sh )    
        call Done(  V_sh )    
        ! history ...    
        call Init( u_tmi, 'u', 'm/s', status, (/vo_tmi,div_tmi/) )
        call Init( v_tmi, 'u', 'm/s', status, (/vo_tmi,div_tmi/) )
        call AddHistory( u_tmi, 'VoD to UV;;interpol to gg', status )
        call AddHistory( v_tmi, 'VoD to UV;;interpol to gg', status )

        ! clear        
        call Done( VO_sh )  ;  deallocate( VO_sh )
        call Done(  D_sh )  ;  deallocate(  D_sh )

        ! remove cos(lat) factor:
        do j = 1, ggi%nlat
          i1 = ggi%i1(j)
          i2 = ggi%im(j)
          u_gg(i1:i2,1:nlev) = u_gg(i1:i2,1:nlev) / cos(ggi%lat(j))
          v_gg(i1:i2,1:nlev) = v_gg(i1:i2,1:nlev) / cos(ggi%lat(j))
        end do
        call AddHistory( u_tmi, 'divide by cos(lat)', status )
        call AddHistory( v_tmi, 'divide by cos(lat)', status )
        
        ! ~~~

        select case ( sourcetype )
        
          case ( 'ecmwf', 'msc-data' )
        
            call wrtgol( 'tmm read  : Q    ', t1, '  -  ', t2 ); call goPr

            ! read humidity
            call FillBuffer( tmm, archivekey, 'Q', tday, t1, t2, 'n', 'n', status )
            IF_NOTOK_RETURN(status=1)

              ! check ...
              call CheckBuffer( tmm, status, gridtype='gg', np=ggi%np, nlev=nlev )
              IF_NOTOK_RETURN(status=1)

              ! copy 3d gg field from buffer; copy info:
              Q_gg(:,1:nlev) = tmm%buf_gg     ! kg h2o / kg air

              ! info ...
              call Init( Q_tmi, tmm%buf_tmi, status )

              ! copy surface pressure:
              sp_gg  = tmm%buf_sp_gg      ! Pa
              call Init( sp_tmi, 'sp', 'Pa', status, (/tmm%buf_tmi/) )

            call wrtgol( 'tmm read  : T   ', t1, '  -  ', t2 ); call goPr

            ! read T
            call FillBuffer( tmm, archivekey, 'T', tday, t1, t2, 'n', 'n', status )
            IF_NOTOK_RETURN(status=1)

              ! check ...
              call CheckBuffer( tmm, status, gridtype='sh', shT=shT, nlev=nlev )
              IF_NOTOK_RETURN(status=1)

              ! convert from sh to gg
              do l = 1, nlev
                call Interpol( tmm%buf_shi, tmm%buf_sh(:,l), ggi, T_gg(:,l), status )
                IF_NOTOK_RETURN(status=1)
              end do

              ! info ...
              call Init( T_tmi, tmm%buf_tmi, status )
              call AddHistory( T_tmi, 'interpol to gg', status )


          case ( 'ncep-cdc', 'ncep-gfs' )
        
            call wrtgol( 'tmm read  : Q    ', t1, '  -  ', t2 ); call goPr

            ! read humidity
            call FillBuffer( tmm, archivekey, 'Q', tday, t1, t2, 'n', 'n', status )
            IF_NOTOK_RETURN(status=1)

              ! check ...
              call CheckBuffer( tmm, status, gridtype='sh', np=ggi%np, nlev=nlev )
              IF_NOTOK_RETURN(status=1)

              ! check ...
              call CheckBuffer( tmm, status, gridtype='sh', shT=shT, nlev=nlev )
              IF_NOTOK_RETURN(status=1)

              ! convert from sh to gg
              do l = 1, nlev
                call Interpol( tmm%buf_shi, tmm%buf_sh(:,l), ggi, Q_gg(:,l), status )   ! kg h2o / kg air
                IF_NOTOK_RETURN(status=1)
              end do

              ! prevent negatives ...
              Q_gg = max( 0.0, Q_gg )

              ! info ...
              call Init( Q_tmi, tmm%buf_tmi, status )
              call AddHistory( Q_tmi, 'interpol to gg', status )
              call AddHistory( Q_tmi, 'truncate', status )

              ! interpolate surface pressure:
              call Interpol( tmm%buf_shi, tmm%buf_lnsp_sh, ggi, sp_gg, status )   ! ln(Pa)
              IF_NOTOK_RETURN(status=1)
              sp_gg = exp( sp_gg )        ! Pa
              ! info ...
              call Init( sp_tmi, 'sp', 'Pa', status, (/tmm%buf_tmi/) )

            call wrtgol( 'tmm read  : T   ', t1, '  -  ', t2 ); call goPr

            ! read virtual temperature
            call FillBuffer( tmm, archivekey, 'Tv', tday, t1, t2, 'n', 'n', status )
            IF_NOTOK_RETURN(status=1)

              ! check ...
              call CheckBuffer( tmm, status, gridtype='sh', shT=shT, nlev=nlev )
              IF_NOTOK_RETURN(status=1)

              ! convert from sh to gg
              do l = 1, nlev
                call Interpol( tmm%buf_shi, tmm%buf_sh(:,l), ggi, T_gg(:,l), status )
                IF_NOTOK_RETURN(status=1)
              end do
              
              ! convert from virtual to normal temperature:
              T_gg = RealTemperature( T_gg, Q_gg )

              ! info ...
              call Init( T_tmi, tmm%buf_tmi, status )
              call AddHistory( T_tmi, 'interpol to gg', status )


          case default

            write (gol,'("unsupported source type `",a,"` for raw surface fields")') trim(sourcetype); call goErr
            write (gol,'("in ",a)') rname; call goErr; status=1; return

        end select
        
        
        ! ~~~

        write (gol,'("    interpol : W")'); call goPr
        
        ! omega on gg and TM levels:
        allocate( tmp_gg(ggi%np,levi%nlev+1) )
        
        ! convert from ll to gg:
        do l = 1, levi%nlev+1
          call Interpol( lli, omega(:,:,l), ggi, tmp_gg(:,l), status )
          IF_NOTOK_RETURN(status=1)
        end do
        
        ! convert to parent levels (buffer) from TM levels:
        call FillLevels( tmm%buf_levi, 'w', sp_gg, w_gg, &
                         levi, tmp_gg, 'bottom', status )
        IF_NOTOK_RETURN(status=1)

        ! info ...
        call Init( w_tmi, omega_tmi, status )
        call AddHistory( w_tmi, 'interpol to gg and raw levels', status )
        
        ! clear
        deallocate( tmp_gg )


        !
        ! ensure that layers are in ecmwf order (top->down)
        !
        
        select case ( sourcetype )
        
          case ( 'ecmwf', 'msc-data' )
          
            ! copy level info from buffer:
            call Init( leviX, tmm%buf_levi%nlev, tmm%buf_levi%a, tmm%buf_levi%b, status )
            IF_NOTOK_RETURN(status=1)
            
          case ( 'ncep-cdc', 'ncep-gfs' )
          
            ! revert level info from buffer:
            allocate( aX(0:tmm%buf_levi%nlev) )
            allocate( bX(0:tmm%buf_levi%nlev) )
            do l = 0, tmm%buf_levi%nlev
              aX(l) = tmm%buf_levi%a(tmm%buf_levi%nlev-l)
              bX(l) = tmm%buf_levi%b(tmm%buf_levi%nlev-l)
            end do
            call Init( leviX, tmm%buf_levi%nlev, aX, bX, status )
            IF_NOTOK_RETURN(status=1)
            deallocate( aX )
            deallocate( bX )
            
            ! revert fields
            allocate( tmp_gg(ggi%np,0:nlev) )
            tmp_gg = u_gg
            do l = 1, nlev
              u_gg(:,l) = tmp_gg(:,nlev+1-l)
            end do
            tmp_gg = v_gg
            do l = 1, nlev
              v_gg(:,l) = tmp_gg(:,nlev+1-l)
            end do
            tmp_gg = w_gg
            do l = 1, nlev
              w_gg(:,l) = tmp_gg(:,nlev+1-l)
            end do
            tmp_gg = t_gg
            do l = 1, nlev
              t_gg(:,l) = tmp_gg(:,nlev+1-l)
            end do
            tmp_gg = q_gg
            do l = 1, nlev
              q_gg(:,l) = tmp_gg(:,nlev+1-l)
            end do
            deallocate( tmp_gg )

        end select
        

        !
        ! updraughts/downdraughts
        !

        write (gol,'("    tmm updr/downdr")'); call goPr

        !
        ! WARNING: the TMPP2/tmpp_conv-gg on bsgi59 is probably not bugfree:
        !  o oro not read
        !  o pw = w/g [kg/m2/s], but tmpp_conv_tiedtke expects Pa/s ?
        !  o factor to convert from lshf to evaporation
        !      here  :  1 m/s = 2.45e9 W/m2 (at 20 degrees C)
        !      binas :  lvap = 2.5 e6 J/kg
        !      binas :  Lc   = 2.26e6 J/kg  (at 0 deg C)
        !
        ! There are however good reasons to step over to the conv-gg code
        ! by Dirk Olivie
        !  o no unnecessary interpolations to half levels
        !  o removed stuff that was based on the ec 19 levels
        !
        
        ! extra fields:
        allocate( ggmask(ggi%np) )
        allocate(  qam_gg(ggi%np,0:nlev) )
        allocate(  qac_gg(ggi%np,0:nlev) )
        allocate( entu_gg(ggi%np,nlev) )
        allocate( detu_gg(ggi%np,nlev) )
        allocate( entd_gg(ggi%np,nlev) )
        allocate( detd_gg(ggi%np,nlev) )
        
        ! Set mask for averaging over ll;
        ! one extra row to compute derivatives.
        ! This routine changes ggi: flag for each row to be processed or not
        call InterpolMask( ggi, ggmask, lli, 1 )

        ! calculate geopotential (z)
        allocate( p_hlev(0:nlev) )
        do i = 1, ggi%np
          ! skip if not required for ll grid:
          if ( .not. ggmask(i) ) cycle
          ! compute pressure at half levels (surf -> top):
          call phlev_ec_rev( nlev, leviX%a, leviX%b, sp_gg(i), p_hlev )
          ! compute z for single column:
          call GeoPot( nlev, oro_gg(i), T_gg(i,1:nlev), Q_gg(i,1:nlev), &
                                                    p_hlev, z_gg(i,1:nlev) )
        end do
        deallocate( p_hlev )

        ! interpolate variables from the center of parent-model layers to the
        ! boundaries of parent-model layers and save result in same memory location ..
        call mid2bound_uv( nlev, ggi%np, u_gg, v_gg, sp_gg, ggmask, leviX%a, leviX%b )
        call mid2bound_t ( nlev, ggi%np, t_gg, sp_gg, ggmask, leviX%a, leviX%b )
        call mid2bound_q ( nlev, ggi%np, q_gg, sp_gg, ggmask, leviX%a, leviX%b, t_gg )
        call mid2bound_z ( nlev, ggi%np, z_gg, sp_gg, ggmask, leviX%a, leviX%b, oro_gg )
       
        ! already on half levels, since interpolated from omega ...        
        !!call mid2bound_w ( nlev, ggi%np, w_gg, sp_gg, ggmask, leviX%a, leviX%b )

        ! NOTE: p is not filled on input, but filled with pressures on output
        call mid2bound_p ( nlev, ggi%np, p_gg, sp_gg, ggmask, leviX%a, leviX%b )

        ! divergence fields
        do l = 0, nlev
          call Divergence( ggi, q_gg(:,l)*u_gg(:,l), q_gg(:,l)*v_gg(:,l), qac_gg(:,l) )
          call Divergence( ggi,           u_gg(:,l),           v_gg(:,l), qam_gg(:,l) )
        end do

        ! loop over gg points:
        do i = 1, ggi%np
        
          ! skip if not required for ll grid:
          if ( .not. ggmask(i) ) cycle

          ! Convert from SLHF (W/m2*interval) to EVAP (m/s)
          ! 1 m/s = 2.45e9 W/m2 (at 20 degrees C). 
          ! Don't forget to change sign from latent heatflux to evaporation!!! 
          evap = - slhf_gg(i) / dt_sec / 2.45e9    ! m/s

          ! only if required for ll:
          call subscal( nlev, leviX%a, leviX%b, &
                        z_gg(i,:), p_gg(i,:), w_gg(i,:), t_gg(i,:), &
                        q_gg(i,:), qac_gg(i,:), qam_gg(i,:), 1.e3*evap, &
                        entu_gg(i,:), detu_gg(i,:), entd_gg(i,:), detd_gg(i,:) )

        end do   ! loop over gg cells

        ! clear
        deallocate( ggmask )
        deallocate( slhf_gg )
        deallocate(  oro_gg )
        deallocate(    u_gg )
        deallocate(    v_gg )
        deallocate(    w_gg )
        deallocate(    t_gg )
        deallocate(    q_gg )
        deallocate(  qam_gg )
        deallocate(  qac_gg )
        deallocate(    p_gg )
        deallocate(    z_gg )

        ! history        
        call Init( entu_tmi, 'entu', 'kg/m2/s', status, &
                     (/sp_tmi,slhf_tmi,oro_tmi,T_tmi,Q_tmi,u_tmi,v_tmi,w_tmi/) )
        call Init( entd_tmi, 'entd', 'kg/m2/s', status, &
                     (/sp_tmi,slhf_tmi,oro_tmi,T_tmi,Q_tmi,u_tmi,v_tmi,w_tmi/) )
        call Init( detu_tmi, 'detu', 'kg/m2/s', status, &
                     (/sp_tmi,slhf_tmi,oro_tmi,T_tmi,Q_tmi,u_tmi,v_tmi,w_tmi/) )
        call Init( detd_tmi, 'detd', 'kg/m2/s', status, &
                     (/sp_tmi,slhf_tmi,oro_tmi,T_tmi,Q_tmi,u_tmi,v_tmi,w_tmi/) )
        
        !
        ! convert from gg to ll
        !
        
        ! determine fraction
        call Init( gg2ll, ggi, lli, status )
        IF_NOTOK_RETURN(status=1)

        ! take fractions of overlapping cells
        call FracSum( gg2ll, sp_gg, sp, status, 'area-aver' )
        IF_NOTOK_RETURN(status=1)
        ! clear
        deallocate( sp_gg )

        ! full level output:
        allocate( llX(lli%nlon,lli%nlat,nlev     ) )
        allocate( ll (lli%nlon,lli%nlat,levi%nlev) )

        ! take fractions of overlapping cells
        do l = 1, nlev
          call FracSum( gg2ll, entu_gg(:,l), llX(:,:,l), status, 'area-aver' )
          IF_NOTOK_RETURN(status=1)
        end do
        ! integrated variables  might become slightly negative ...
        llX = max( 0.0, llX )
        ! combine levels from llX to ll:
        call FillLevels( levi, 'n', sp, ll, leviX, llX, 'sum', status )
        IF_NOTOK_RETURN(status=1)
        ! truncate to number of output levels:
        entu = ll(:,:,1:lout)
        ! info ..
        call AddHistory( entu_tmi, 'gg to ll, area aver;;sum levels', status )

        ! take fractions of overlapping cells
        do l = 1, nlev
          call FracSum( gg2ll, entd_gg(:,l), llX(:,:,l), status, 'area-aver' )
          IF_NOTOK_RETURN(status=1)
        end do
        ! integrated variables  might become slightly negative ...
        llX = max( 0.0, llX )
        ! combine levels from llX to ll:
        call FillLevels( levi, 'n', sp, ll, leviX, llX, 'sum', status )
        IF_NOTOK_RETURN(status=1)
        ! truncate to number of output levels:
        entd = ll(:,:,1:lout)
        ! info ..
        call AddHistory( entd_tmi, 'gg to ll, area aver;;sum levels', status )

        ! take fractions of overlapping cells
        do l = 1, nlev
          call FracSum( gg2ll, detu_gg(:,l), llX(:,:,l), status, 'area-aver' )
          IF_NOTOK_RETURN(status=1)
        end do
        ! integrated variables  might become slightly negative ...
        llX = max( 0.0, llX )
        ! combine levels from llX to ll:
        call FillLevels( levi, 'n', sp, ll, leviX, llX, 'sum', status )
        IF_NOTOK_RETURN(status=1)
        ! truncate to number of output levels:
        detu = ll(:,:,1:lout)
        ! info ..
        call AddHistory( detu_tmi, 'gg to ll, area aver;;sum levels', status )

        ! take fractions of overlapping cells
        do l = 1, nlev
          call FracSum( gg2ll, detd_gg(:,l), llX(:,:,l), status, 'area-aver' )
          IF_NOTOK_RETURN(status=1)
        end do
        ! integrated variables  might become slightly negative ...
        llX = max( 0.0, llX )
        ! combine levels from llX to ll:
        call FillLevels( levi, 'n', sp, ll, leviX, llX, 'sum', status )
        IF_NOTOK_RETURN(status=1)
        ! truncate to number of output levels:
        detd = ll(:,:,1:lout)
        ! info ..
        call AddHistory( detd_tmi, 'gg to ll, area aver;;sum levels', status )

        ! clear
        call Done( gg2ll )
        deallocate( entu_gg )
        deallocate( entd_gg )
        deallocate( detu_gg )
        deallocate( detd_gg )
        deallocate( llX )
        deallocate( ll  )
        call Done( leviX, status )
        IF_NOTOK_RETURN(status=1)


        !
        ! done
        !
        
        call Done( ggi, status )
        IF_NOTOK_RETURN(status=1)


      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_Read_Convec


  ! ==================================================================
  ! ===
  ! ===  Olsson surface roughness
  ! ===
  ! ==================================================================


  subroutine tmm_Read_SR_OLS( tmm, archivekey, tday, t1, t2, &
                                lli, ll, tmi, status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, Get, wrtgol
    use GO      , only : goSplitLine
    use Grid    , only : TllGridInfo, Init, Done, Interpol
    use tmm_info, only : TMeteoInfo, Init, AddHistory

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)     ::  tmm
    character(len=*), intent(in)      ::  archivekey
    type(TDate), intent(in)           ::  tday, t1, t2
    type(TllGridInfo), intent(in)     ::  lli
    real, intent(out)                 ::  ll(:,:)
    type(TMeteoInfo), intent(out)     ::  tmi
    integer, intent(out)              ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_Read_SR_OLS'
    
    ! grid size
    integer, parameter  ::  nlon = 361, nlat = 181
    
    ! --- local -------------------------------
    
    character(len=10)         ::  sourcetype
    character(len=256)        ::  sourcename

    integer                   ::  month
    character(len=256)        ::  fname
    logical                   ::  exist, opened
    integer                   ::  fu
    type(TllGridInfo)         ::  lli_ols
    real                      ::  sr_ols(nlon,nlat)
    integer                   ::  i, j

    ! --- begin -------------------------------
    
    ! split source key in type and name:
    call goSplitLine( archivekey, sourcetype, ':', sourcename, status )
    IF_NOTOK_RETURN(status=1)
    
    ! input TMPP fields or raw prism fields ?
    select case ( sourcetype )
    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! read directly from hdf file
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      case ( 'tmpp', 'tm5' )

        ! read from surf file:      
        call ReadField( tmm, archivekey, 'srols', tday, t1, t1, &
                             lli, 'n', ll, tmi, status )
        IF_NOTOK_RETURN(status=1)

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! convert raw data:
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'ecmwf', 'ncep-cdc', 'ncep-gfs' )
      
        call wrtgol( 'tmm read  : srols ', t1, '  -  ', t2 ); call goPr

        ! month
        call Get( t1, month=month )

        ! write filename:
        write (fname,'(a,"/OLSSON-SR_OLSSON_360_180_",i2.2,".d")') trim(tmm%input_dir), month

        ! exist ?
        inquire( file=fname, exist=exist )
        if ( .not. exist ) then
          write (gol,'("Olsson SR file not found:")'); call goErr
          write (gol,'("  ",a)') trim(fname); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! select free file unit:
        fu = 1234
        do
          inquire( unit=fu, opened=opened )
          if ( .not. opened ) exit
          fu = fu + 1
        end do

        ! open data file:
        open( fu, file=trim(fname), form='formatted', status='old', iostat=status )
        if (status/=0) then
          write (gol,'("while opening olsson data file:")'); call goErr
          write (gol,'("  ",a)') trim(fname); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! read field:
        read (fu,*,iostat=status) sr_ols
        if (status/=0) then
          write (gol,'("while reading from olsson data file:")'); call goErr
          write (gol,'("  ",a)') trim(fname); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! close file:
        close( fu, iostat=status )
        if (status/=0) then
          write (gol,'("while closing olsson data file:")'); call goErr
          write (gol,'("  ",a)') trim(fname); call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if

        ! setup grid definition:
        !   lon  :  -180.0  -179.0  ..  180.0    ( 1 deg resolution; 360 points; date line twice)
        !   lat  :   -90.0   -89.0  ..   90.0    ( 1 deg resolution; 180 points; includes poles)
        call Init( lli_ols, -180.00, 360.0/(nlon-1), nlon, -90.0, 180.0/(nlat-1), nlat, status )
        IF_NOTOK_RETURN(status=1)

        ! interpol
        do j = 1, lli%nlat
          do i = 1, lli%nlon
            call Interpol( lli_ols, sr_ols, lli%lon_deg(i), lli%lat_deg(j), ll(i,j) )
          end do
        end do

        ! done
        call Done( lli_ols, status )
        IF_NOTOK_RETURN(status=1)
    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error ...
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case default
      
        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
        
    end select
    
    ! ok
    status = 0
    
  end subroutine tmm_Read_SR_OLS


!  ! ==================================================================
!  ! ===
!  ! ===  pv/theta -> eqv.lat.
!  ! ===
!  ! ==================================================================
!
!
!  subroutine tmm_ReadEqvLat( tmm, archivekey, &
!                                  tday, t1, t2, &
!                                  lli, levi, &
!                                  sp, pv, theta, eqvlatb, eqvinds, &
!                                  status )
!  
!    use GO, only : TDate, operator(-), operator(+), operator(/), goSplitLine
!    use Grid, only : TllGridInfo, TLevelInfo
!    
!    use tmm_mf  , only : ReadEqvLatStuff
!    use tmm_info, only : TMeteoInfo
!
!    ! --- in/out --------------------------------
!  
!    type(TTmMeteo), intent(inout)           ::  tmm
!
!    character(len=*), intent(in)            ::  archivekey
!
!    type(TDate), intent(in)                 ::  tday, t1, t2
!
!    type(TllGridInfo), intent(in)           ::  lli
!    type(TLevelInfo), intent(in)            ::  levi      
!
!    real, intent(out)                       ::  sp(:,:)       ! Pa
!    real, intent(out)                       ::  pv(:,:,:)
!    real, intent(out)                       ::  theta(:,:,:)
!    real, intent(out)                       ::  eqvlatb(:,:)
!    integer, intent(out)                    ::  eqvinds(:,:,:)
!
!    integer, intent(out)                    ::  status
!    
!    ! --- const --------------------------------------
!    
!    character(len=*), parameter  ::  rname = mname//'/tmm_ReadEqvLat'
!    
!    ! --- local -------------------------------
!    
!    character(len=10)        ::  sourcetype
!    character(len=256)       ::  sourcename
!    
!    integer                  ::  imf
!    
!    type(TMeteoInfo)         ::  tmi
!    
!    ! --- begin -------------------------------
!    
!    ! split source key in type and name:
!    call goSplitLine( archivekey, sourcetype, ':', sourcename )
!    
!    ! input TMPP fields or raw prism fields ?
!    select case ( sourcetype )
!      
!      case ( 'tmpp' )
!      
!        ! pv valid for [t1,t2]
!        call ReadField( tmm, archivekey, 'PVo', tday, t1, t2, &
!                             lli, 'n', levi, 'n', sp, pv, tmi, status )
!        IF_NOTOK_RETURN(status=1)
!    
!        ! theta valid for [t1,t2]
!        call ReadField( tmm, archivekey, 'theta', tday, t1, t2, &
!                             lli, 'n', levi, 'n', sp, theta, tmi, status )
!        IF_NOTOK_RETURN(status=1)
!    
!        !
!        ! eqv lat bounds and indices
!        !
!        ! select (eventually retrieve first) the meteo file with this param:
!        call SelectMF( tmm, 'i', archivekey, 'eqvlatb', tday, t1, t2, imf, status )
!        IF_NOTOK_RETURN(status=1)
!        !
!        ! read from selected file
!        call ReadEqvLatStuff( tmm%mf(imf), t1, t2, eqvlatb, eqvinds, status )
!        IF_NOTOK_RETURN(status=1)
!    
!      case default
!      
!        write (gol,'("unsupported source type `",a,"`")') trim(sourcetype)
!        write (gol,'("in ",a)') rname; call goErr; status=1; return
!        
!    end select
!    
!    ! ok
!    status = 0
!    
!  end subroutine tmm_ReadEqvLat


  ! ##########################################################################################
  ! ###
  ! ### output
  ! ###
  ! ##########################################################################################
  

  !
  ! call WriteField( tmmd, 'od-fc-ml60-glb3x2', 'T', 'K', tday, t1, t2, &
  !                        lli, 'n', sp, status )
  !

  subroutine tmm_WriteField_2d( tmm, archivekey, &
                                     tmi, paramkey, unit, tday, t1, t2, &
                                     lli, nuv, ll, status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : TDate, wrtgol
    use Grid    , only : TllGridInfo
    use tmm_mf  , only : WriteRecord
    use tmm_info, only : TMeteoInfo

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    character(len=*), intent(in)            ::  archivekey
    type(TMeteoInfo), intent(in)            ::  tmi
    character(len=*), intent(in)            ::  paramkey
    character(len=*), intent(in)            ::  unit
    type(TDate), intent(in)                 ::  tday, t1, t2
    type(TllGridInfo), intent(in)           ::  lli
    character(len=1), intent(in)            ::  nuv
    real, intent(out)                       ::  ll(:,:) 
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_WriteField_2d'
    
    ! --- local ----------------------------------------
    
    integer          ::  imf
    
    ! --- begin ----------------------------------
    
    call wrtgol( 'tmm write : '//trim(paramkey)//' ', t1, '  -  ', t2 ); call goPr

    ! check shape of grid:
    if ( ((nuv == 'n') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'u') .and. ((size(ll,1) /= lli%nlon+1) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'v') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat+1))) ) then
      write (gol,'("2d array does not mach with grid definition:")'); call goErr
      write (gol,'("  param  : ",a          )') paramkey   ; call goErr
      write (gol,'("  lli    : ",i3," x ",i3)') lli%nlon, lli%nlat; call goErr
      write (gol,'("  nuv    : ",a          )') nuv; call goErr
      write (gol,'("  ll     : ",i3," x ",i3)') shape(ll); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if

    ! select index of already open meteo file or setup access to new one;
    call SelectMF( tmm, 'o', archivekey, paramkey, tday, t1, t2, imf, status )
    IF_NOTOK_RETURN(status=1)

    ! write
    call WriteRecord( tmm%mf(imf), tmi, paramkey, unit, tday, t1, t2, &
                        lli, nuv, ll, status )
    IF_NOTOK_RETURN(status=1)

    ! ok
    status = 0
    
  end subroutine tmm_WriteField_2d
  
  
  !
  ! call WriteField( tmmd, 'od-fc-ml60-glb3x2', 'T', 'K', tday, t1, t2, &
  !                        lli, 'n', levi, spm, temper, status )
  !

  subroutine tmm_WriteField_3d( tmm, archivekey, &
                                     tmi, spname, paramkey, unit, tday, t1, t2, &
                                     lli, nuv, levi, nw, sp, ll, status )
  
    use GO      , only : goErr, gol, goPr
    use GO      , only : gol, goPr
    use GO      , only : TDate, wrtgol
    use Grid    , only : TllGridInfo, TLevelInfo
    use tmm_mf  , only : WriteRecord
    use tmm_info, only : TMeteoInfo

    ! --- in/out --------------------------------
  
    type(TTmMeteo), intent(inout)           ::  tmm
    character(len=*), intent(in)            ::  archivekey
    type(TMeteoInfo), intent(in)            ::  tmi
    character(len=*), intent(in)            ::  spname
    character(len=*), intent(in)            ::  paramkey
    character(len=*), intent(in)            ::  unit
    type(TDate), intent(in)                 ::  tday, t1, t2
    type(TllGridInfo), intent(in)           ::  lli
    character(len=1), intent(in)            ::  nuv
    type(TLevelInfo), intent(in)            ::  levi      
    character(len=1), intent(in)            ::  nw
    real, intent(in)                        ::  sp(:,:)        ! Pa
    real, intent(out)                       ::  ll(:,:,:) 
    integer, intent(out)                    ::  status
    
    ! --- const ------------------------------------
    
    character(len=*), parameter ::  rname = mname//'/tmm_WriteField_3d'
    
    ! --- local ----------------------------------------
    
    integer          ::  imf
    
    ! --- begin ----------------------------------
    
    call wrtgol( 'tmm write : '//trim(paramkey)//' ', t1, '  -  ', t2 ); call goPr

    ! check shape of grid:
    if ( ((nuv == 'n') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'u') .and. ((size(ll,1) /= lli%nlon+1) .or. (size(ll,2) /= lli%nlat  ))) .or. &
         ((nuv == 'v') .and. ((size(ll,1) /= lli%nlon  ) .or. (size(ll,2) /= lli%nlat+1))) .or. &
         ((nuv == 'n') .and. ((size(sp,1) /= lli%nlon  ) .or. (size(sp,2) /= lli%nlat  ))) .or. &
         ((nuv == 'u') .and. ((size(sp,1) /= lli%nlon+1) .or. (size(sp,2) /= lli%nlat  ))) .or. &
         ((nuv == 'v') .and. ((size(sp,1) /= lli%nlon  ) .or. (size(sp,2) /= lli%nlat+1))) .or. &
         ((nw  == 'n') .and. (size(ll,3) > levi%nlev  )) .or. &
         ((nw  == 'w') .and. (size(ll,3) > levi%nlev+1)) ) then
      write (gol,'("3d arrays do not match with grid definition:")'); call goErr
      write (gol,'("  param  : ",a          )') paramkey; call goErr
      write (gol,'("  lli    : ",i3," x ",i3         )') lli%nlon, lli%nlat; call goErr
      write (gol,'("  nuv    : ",a                   )') nuv; call goErr
      write (gol,'("  levi   : ",i3                  )') levi%nlev; call goErr
      write (gol,'("  nw     : ",a                   )') nw; call goErr
      write (gol,'("  sp     : ",i3," x ",i3         )') shape(sp); call goErr
      write (gol,'("  ll     : ",i3," x ",i3," x ",i3)') shape(ll); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if

    ! select index of already open meteo file or setup access to new one;
    call SelectMF( tmm, 'o', archivekey, paramkey, tday, t1, t2, imf, status )
    IF_NOTOK_RETURN(status=1)

    ! write
    call WriteRecord( tmm%mf(imf), tmi, spname, paramkey, unit, tday, t1, t2, &
                        lli, nuv, levi, nw, sp, ll, status )
    IF_NOTOK_RETURN(status=1)

    ! ok
    status = 0
    
  end subroutine tmm_WriteField_3d
  
  
  

end module TMM
