!#################################################################
!
! contains calls to user-specific output routines, e.g.
! instantaneous mix files, station output, output of flight tracks etc.
!
!### macro's #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module user_output

  use GO, only : gol, goErr

  implicit none

  ! --- in/out ------------------------

  private

  public :: user_output_init
  public :: user_output_step
  public :: user_output_mean
  public :: user_output_done

  public :: flight_data
  public :: column_data
  public :: column3d_data
  public :: eddyflux_data
  public :: station_data
  public :: mix_data
  public :: mmix_data
  public :: flux1x1_data

  ! --- const -------------------------

  character(len=*), parameter  ::  mname = 'user_output'
  
  ! --- var ---------------------------

  logical           :: flight_data = .false.  ! signal for flight output

  integer           :: column_data_dhour=24         ! every dhour hour

  logical           :: column3d_data = .false.  ! signal for columnoutput
  integer           :: column3d_data_dhour=24         ! every dhour hour

  logical           :: eddyflux_data = .false.  ! signal for eddyflux output

  logical           :: station_data = .true.  ! signal for station output

  logical           :: column_data = .true.  ! signal for column output

  logical           :: mix_data = .false.     ! signal for mix output
  integer           :: mix_data_dhour         ! every dhour hour

  logical           :: flux1x1_data = .false.     ! signal for flux1x1 output
  integer           :: flux1x1_data_dhour         ! every dhour hour

  logical           :: mmix_data = .false.     ! signal for mean mix output


contains


  !-------------------------------------------------------------
  ! user_output_init:
  !      Initialise user-specified model output (all regions)
  !-------------------------------------------------------------

  subroutine user_output_init( status )

    use GO,                  only : TrcFile, Init, Done, ReadRc
    use global_data,         only : rcfile
    use user_output_mmix,    only : init_mmix
    use user_output_flux1x1, only : init_flux1x1
    use user_output_station, only : read_stationlist, init_station_output, StationlistFFilename 
    use user_output_column,  only : read_columnlist, init_column_output, columnlistFFilename 
    use user_output_mix,     only : output_mix_init
    use user_output_column3d,only : init_columndata3d
    use user_output_forecast,only : ObslistFFilename
    use user_output_eddyflux, only : read_eddyfluxlist, init_eddyflux_output, eddyfluxlistFFilename 
    
    ! --- in/out -----------------------------
    
    integer, intent(out)      ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/user_output_init'
    
    ! --- local -------------------------------

    type(TrcFile)       :: rcF

    ! --- begin -------------------------------
    
    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'output.station', station_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( station_data ) then
       call ReadRc( rcF, 'input.station.filename', StationlistFFilename, status)
       IF_NOTOK_RETURN(status=1)
       call read_stationlist
       call init_station_output
    end if

    call ReadRc( rcF, 'output.column', column_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( column_data ) then
       call ReadRc( rcF, 'input.column.filename', columnlistFFilename, status)
       IF_NOTOK_RETURN(status=1)
       call read_columnlist
       call init_column_output
    end if

    call ReadRc( rcF, 'output.flight', flight_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'output.column3d', column3d_data, status ,default=.false.)
    IF_NOTOK_RETURN(status=1)
    if ( column3d_data ) then
      call ReadRc( rcF, 'output.column3d.dhour', column3d_data_dhour, status , default=24)
      IF_NOTOK_RETURN(status=1)
      call init_columndata3d
    end if

    call ReadRc( rcF, 'output.eddyflux', eddyflux_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( eddyflux_data ) then
       call ReadRc( rcF, 'input.eddyflux.file',eddyfluxlistFFilename, status)
       IF_NOTOK_RETURN(status=1)
       call read_eddyfluxlist
       call init_eddyflux_output
    end if

    call ReadRc( rcF, 'output.mix', mix_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( mix_data ) then
      call ReadRc( rcF, 'output.mix.dhour', mix_data_dhour, status ,default=24)
      IF_NOTOK_RETURN(status=1)
      call output_mix_init
    end if

    ! Note that the flux1x1 module uses the emissions_averagers code.  That
    ! code was originally written to accumulate weekly average posterior fluxes
    ! during an inverse run, and it is handled with calls scattered throughout the
    ! EnKF modules.  If an inverse run also specifies, output.flux1x1:T in its
    ! rc file, then the weekly average emissions (which, incidentally get written
    ! to the savestate.hdf file, not flux1x1*hdf files) need to be turned off.
    !  - Andy, 15 Nov 08
    call ReadRc( rcF, 'output.flux1x1', flux1x1_data, status ,default=.false.)
    IF_NOTOK_RETURN(status=1)
    if ( flux1x1_data ) then
      call ReadRc( rcF, 'output.flux1x1.dhour', flux1x1_data_dhour, status , default=24)
      IF_NOTOK_RETURN(status=1)
      call init_flux1x1
    end if

    ! initialise accumulation of the mean mixing ratio fields
    call ReadRc( rcF, 'output.mmix', mmix_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( mmix_data ) then
      call init_mmix 
    end if  

    ! close rcfile:
    call Done( rcF, status  )
    IF_NOTOK_RETURN(status=1)

    ! ok
    status = 0

  end subroutine user_output_init


  !-------------------------------------------------------------
  ! user_output_done:
  !      Finalise user-specified model output for the region given
  !-------------------------------------------------------------

  subroutine user_output_done( status )

    use user_output_mmix,     only : write_mmix, free_mmix
    use user_output_station,  only : close_stationfiles, write_stationfiles
    use user_output_column,  only : close_columnfiles, write_columnfiles
    use user_output_eddyflux, only : close_eddyfluxfiles, write_eddyfluxfiles
    use user_output_mix,      only : output_mix_close
    use user_output_column3d,   only : free_columndata3d  
    use dims,                 only : nregions
    
    ! --- in/out -----------------------------
    
    integer, intent(out)      ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/user_output_done'
    
    ! --- begin -----------------------------
    
    ! write the mean mixing ratio fields to file
    if ( mmix_data ) then
      call write_mmix
      call free_mmix
    end if

    if(column3d_data ) call free_columndata3d 
    
    if ( write_stationfiles .and. station_data) call close_stationfiles

    if ( write_columnfiles .and. column_data) call close_columnfiles

    if ( write_eddyfluxfiles .and. eddyflux_data) call close_eddyfluxfiles

    if ( mix_data ) call output_mix_close

    ! ok
    status = 0

  end subroutine user_output_done

  SUBROUTINE user_output_mean

    use dims,                     only: itau, ndyn_max
    use dims,                     only: nregions
    use dims,                     only: revert
    use dims,                     only: newmonth, newday

    use datetime,                 only: tau2date

    use user_output_column3d,     only: evaluate_columndata3d
    use user_output_column3d,     only: write_columndata3d
    use user_output_column3d,     only: reset_columndata_accumulator3d
    use user_output_station,      only: evaluate_stationconc
    use user_output_station,      only: reset_stationconc_accumulator
    use user_output_station,      only: write_stationconc

    use user_output_column,      only: evaluate_columnconc
    use user_output_column,      only: reset_columnconc_accumulator
    use user_output_column,      only: write_columnconc

    use user_output_flux1x1,      only: accumulate_flux1x1
    use user_output_flux1x1,      only: reset_flux1x1
    use user_output_flux1x1,      only: write_flux1x1

    use user_output_eddyflux,      only: evaluate_eddyflux
    use user_output_eddyflux,      only: reset_eddyflux_accumulator
    use user_output_eddyflux,      only: write_eddyflux
 

    use user_output_mmix,         only: reset_mmix
    use user_output_mmix,         only: write_mmix

    use user_output_mix,          only: write_mix


    IMPLICIT NONE

    !__IO___________________________________________________________________

    !__LOCAL_VARIABLES______________________________________________________

    INTEGER ::  region,statuss
    integer,dimension(6)            :: idate_f

    !__START_SUBROUTINE______________________________________________________

    call tau2date(itau,idate_f)

    IF(station_data)THEN
      IF(mod(itau, ndyn_max) == 0) THEN
         CALL evaluate_stationconc
         CALL write_stationconc
         CALL reset_stationconc_accumulator  
      ENDIF
    ENDIF

    IF(column_data)THEN
      IF(mod(itau, ndyn_max) == 0) THEN
         CALL evaluate_columnconc
         CALL write_columnconc
         CALL reset_columnconc_accumulator  
      ENDIF
    ENDIF

    IF(eddyflux_data)THEN
       IF(mod(itau, ndyn_max) == 0) THEN
          CALL evaluate_eddyflux
          CALL write_eddyflux
          CALL reset_eddyflux_accumulator  
       ENDIF
    ENDIF
    
    IF(column3d_data)THEN
       do region=nregions,1,-1
          if ( modulo(idate_f(4),column3d_data_dhour) == 0 ) then
             CALL evaluate_columndata3d(region)
             CALL write_columndata3d(region,statuss)
             CALL reset_columndata_accumulator3d(region)    
          endif
       enddo
    ENDIF
    
    IF(flux1x1_data)THEN
       if ( modulo(idate_f(4),flux1x1_data_dhour) == 0 ) then
          CALL write_flux1x1(flux1x1_data_dhour)
          CALL reset_flux1x1
       endif
    ENDIF

    IF ( mix_data) THEN
       do region=nregions,1,-1
          if ( modulo(idate_f(4),mix_data_dhour) == 0 ) then
             call write_mix(region)
          endif
       enddo
    ENDIF
    

    !===============
    ! mmix output
    !===============
!cmk: this creates double writing of mmix at end of monthly run.....removed!
!cmk: double writing means loss of data.....
!cmk     IF(mmix_data)THEN
!cmk 
!cmk         IF(newmonth) THEN
!cmk !        IF(newday) THEN
!cmk           CALL write_mmix
!cmk           CALL reset_mmix
!cmk         ENDIF
!cmk        
!cmk     ENDIF



  END SUBROUTINE  

!===========================================================================================================


  !-------------------------------------------------------------
  ! user_output_step:
  !      Define user-specified model output for the region given
  !      Called every time step
  !-------------------------------------------------------------

  subroutine user_output_step( region, status )

    use dims,                 only : itaur
    use datetime,             only : tau2date
    use user_output_station,  only : output_stationconc  
    use user_output_column,  only : output_columnconc  
    use user_output_eddyflux,  only : output_eddyflux  
    use user_output_column3d,   only : output_columndata3d  
    use user_output_flight,   only : get_flightdata
    use user_output_mmix,     only : accumulate_mmix
    use user_output_mix,      only : accumulate_mix
    use user_output_flux1x1,      only: accumulate_flux1x1
    use user_output_forecast, only : get_forecast

    ! --- in/out ------------------------------
    
    integer, intent(in)       ::  region
    integer, intent(out)      ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/user_output_step'
    
    ! --- local ------------------------------
    
    integer,dimension(6)            :: idate_f

    ! --- begin ------------------------------

    call tau2date(itaur(region),idate_f)

    if ( mmix_data ) call accumulate_mmix( region )

    if ( station_data ) call output_stationconc(region)

    if ( column_data ) call output_columnconc(region)

    if ( eddyflux_data ) call output_eddyflux(region)

    if ( column3d_data ) call output_columndata3d(region)

    if ( flight_data )  call get_flightdata(region,idate_f)

    ! The second keyword here ("force") is for instantaneous sampling.  Default
    ! value of .false. yields a four-hour window centered on sample time.
    !
    call get_forecast(region,itaur(region),.false.)

    if ( flux1x1_data ) call accumulate_flux1x1

    if ( mix_data ) call accumulate_mix(region)

    ! ok
    status = 0

  end subroutine user_output_step


end module user_output
